Package: marsrad
Title: Mars Solar Radiation
Version: 1.0.1
Authors@R: 
    person(given = "Georges",
           family = "Labrèche",
           role = c("aut", "cre"),
           email = "georges@tanagraspace.com",
           comment = c(ORCID = "0000-0002-2104-8789"))
Description: A set of functions to calculate solar irradiance and insolation on Mars horizontal and inclined surfaces. Based on NASA Technical Memoranda 102299, 103623, 105216, 106321, and 106700, i.e. the canonical Mars solar radiation papers.
License: GPL-3
URL: https://georges.fyi/marsrad/,
        https://github.com/georgeslabreche/marsrad
BugReports: https://github.com/georgeslabreche/marsrad/issues
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: R (>= 2.10)
Imports: stats
Suggests: testthat, covr, DT, htmltools
NeedsCompilation: no
Packaged: 2026-05-09 22:44:53 UTC; georgeslabreche
Author: Georges Labrèche [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2104-8789>)
Maintainer: Georges Labrèche <georges@tanagraspace.com>
Repository: CRAN
Date/Publication: 2026-05-09 23:00:02 UTC
Built: R 4.6.0; ; 2026-05-09 23:23:10 UTC; unix

# Generated by roxygen2: do not edit by hand

export(G_ali)
export(G_b)
export(G_bh)
export(G_bi)
export(G_dh)
export(G_di)
export(G_h)
export(G_i)
export(G_ob)
export(G_obh)
export(H_ali)
export(H_bh)
export(H_bi)
export(H_dh)
export(H_di)
export(H_h)
export(H_i)
export(H_obh)
export(I_ali)
export(I_bh)
export(I_bi)
export(I_dh)
export(I_di)
export(I_h)
export(I_i)
export(I_obh)
export(T_d)
export(Z)
export(albedo)
export(declination)
export(f)
export(is_irradiated)
export(is_polar_day)
export(is_polar_night)
export(optical_depth)
export(optimal_angle)
export(sunrise)
export(sunset)
importFrom(stats,integrate)

[![R-CMD-check](https://github.com/georgeslabreche/marsrad/workflows/R-CMD-check/badge.svg)](https://github.com/georgeslabreche/marsrad/actions)

# marsrad: Mars Solar Radiation

## Overview

An R package to calculate solar irradiance (instantaneous power) and insolation (energy over time) on the Martian surface for both horizontal and inclined surfaces. This package is essential in designing solar power systems for Mars missions and mission planning. Based on NASA Technical Memoranda 102299, 103623, 105216, 106321, and 106700, i.e. the canonical Mars solar radiation papers.

## Citation

If you use this package in your research or publication, please cite the paper it was developed for:

**Labrèche, Georges and Cordes, Florian.** (2020). "[Using a Rover's Active Suspension System as a 2-Axis Solar Tracker Mechanism](https://www.hou.usra.edu/meetings/isairas2020fullpapers/pdf/5035.pdf)." *15th International Symposium on Artificial Intelligence, Robotics and Automation in Space (i-SAIRAS '20)*, October 2020. https://www.hou.usra.edu/meetings/isairas2020fullpapers/pdf/5035.pdf

### BibTeX

```bibtex
@inproceedings{Labreche2020_iSAIRAS20_MarsRoverSherpaTT,
  title={Using a Rover's Active Suspension System as a 2-Axis Solar Tracker Mechanism},
  author={Labrèche, Georges and Cordes, Florian},
  booktitle={15th International Symposium on Artificial Intelligence, Robotics and Automation in Space (i-SAIRAS '20)},
  month={October},
  year={2020},
  url={https://www.hou.usra.edu/meetings/isairas2020fullpapers/pdf/5035.pdf}
}
```

## See Also

**Labrèche, G**. (2020). [Exploiting the SherpaTT Rover Active Suspension System to Enable Optimal Solar Array Inclination and Orientation for Long Traverses in a Martian Environment](https://www.diva-portal.org/smash/record.jsf?pid=diva2:1413245). *Master's Thesis, Luleå University of Technology*, March 2020. https://www.diva-portal.org/smash/record.jsf?pid=diva2:1413245

## Key Features

- Calculate solar radiation on horizontal and inclined surfaces
- Account for Martian atmospheric conditions including dust opacity
- Support for various surface orientations and slope angles
- Handle polar day/night conditions
- Compute optimal tilt angles for photovoltaic arrays
- Calculate sunrise/sunset times on Mars

## What's Included

The package implements equations for:

- **Global irradiance** - total solar radiation (direct + diffuse + reflected)
- **Direct beam radiation** - sunlight arriving directly from the sun
- **Diffuse radiation** - light scattered by atmospheric dust
- **Albedo reflection** - ground-reflected radiation
- **Insolation** - integrated solar energy over time
- **Utility functions** - sunrise/sunset, optimal angles, polar day/night detection

For a complete list of all functions with descriptions, see [FUNCTIONS.md](https://github.com/georgeslabreche/marsrad/blob/master/FUNCTIONS.md).

## Parameters

Calculate and plot solar radiation on Mars as a function of the following parameters:

- Areocentric Longitude (Ls)
- Planetary Latitude (phi)
- Solar Time (omega)
- Atmospheric Opacity (tau)
- Albedo (al)
- Slope Angle (beta)
- Slope Orientation (gamma)

## Old School Cool

The Appelbaum formulas are based on Viking Lander optical depth measurements from the late 1970s and early 1980s. They're 36+ years old now and Mars science has moved on. More recent work includes:

- [An improved model for available solar energy on Mars: Optimizing solar panel orientation to assess potential spacecraft landing sites](https://doi.org/10.1016/j.asr.2023.04.004) (Kerr et al., 2023).
- [A model to calculate solar radiation fluxes on the Martian surface](https://www.swsc-journal.org/articles/swsc/full_html/2015/01/swsc150027/swsc150027.html) (Vicente-Retortillo et al., 2015).
- [Fast and accurate estimation of solar irradiance on Martian slopes](https://agupubs.onlinelibrary.wiley.com/doi/full/10.1029/2008GL034956) (Spiga et al., 2008).

However, the Appelbaum formulas remain useful for first-order estimates, educational purposes, and quick trade studies. They're well-documented, easy to understand, analytically tractable, and give you reasonable ballpark figures for solar energy budgets on Mars. Just don't use them to design your actual flight hardware without consulting more recent atmospheric models.

## Getting Started

### Installation

Install the package from GitHub using devtools:

```r
# Install devtools if you haven't already
install.packages("devtools")

# Install the mars package
devtools::install_github("georgeslabreche/marsrad")
```

### Basic Usage

```r
library(marsrad)

# Calculate global irradiance on a horizontal surface
# Example: Mars surface at Viking Lander 1 site during northern summer
Ls <- 120           # Areocentric longitude (degrees)
phi <- 22.3         # Latitude (degrees) - VL1 location
longitude <- -48    # Longitude (degrees) - VL1 at 48°W
Ts <- 12            # Solar time (hours) - noon
tau <- 0.4          # Atmospheric optical depth
al <- 0.1           # Albedo

# Global irradiance on horizontal surface (W/m²)
Gh <- G_h(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts, tau = tau, al = al)
print(paste("Global irradiance:", round(Gh, 2), "W/m²"))

# Daily insolation on horizontal surface (Wh/m²-day)
Hh <- H_h(Ls = Ls, phi = phi, longitude = longitude, tau = tau, al = al)
print(paste("Daily insolation:", round(Hh, 2), "Wh/m²-day"))

# Calculate for an inclined surface (e.g., tilted solar panel)
beta <- 30          # Tilt angle (degrees)
gamma_c <- 0        # Surface azimuth angle (degrees, 0 = facing equator)

# Global irradiance on inclined surface (W/m²)
Gi <- G_i(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
          tau = tau, al = al, beta = beta, gamma_c = gamma_c)
print(paste("Inclined surface irradiance:", round(Gi, 2), "W/m²"))

# Find optimal tilt angle for maximum daily insolation
optimal <- optimal_angle(Ls = Ls, phi = phi, unit = 2)
print(paste("Optimal tilt angle:", round(optimal, 2), "degrees"))

# Calculate sunrise and sunset times (in solar hours, 0-24)
sunrise_time <- sunrise(Ls = Ls, phi = phi, unit = 3)
sunset_time <- sunset(Ls = Ls, phi = phi, unit = 3)
print(paste("Sunrise:", round(sunrise_time, 2), "hours"))
print(paste("Sunset:", round(sunset_time, 2), "hours"))
```

## Environment Variables

The package can be configured via environment variables to control calculation methods and behavior:

### NET_FLUX_FUNCTION_TYPE

Controls which implementation of the normalized net flux function to use. Affects all radiation calculations.

**Options:**

- `"polynomial"` (default) - Analytical polynomial expression
  - Mean error: ~0.7%
  - Maximum error: ~7% at zenith angles 80-85° and optical depth > 5
  - Works with any albedo value
  - Recommended for most use cases
- `"lookup_v1"` - Lookup table from NASA TM-102299 (Appelbaum & Flood, 1989)
  - Albedo fixed at 0.1
  - Exact values from original tables
  - Only works with discrete zenith angles: 0°, 10°, 20°, 30°, 40°, 50°, 60°, 70°, 80°, 85°
  - Not recommended for calculations with arbitrary zenith angles from `Z()`
- `"lookup_v2"` - Lookup table from NASA TM-103623 (Appelbaum & Flood, 1990)
  - Supports albedo 0.1 and 0.4
  - Exact values from updated tables
  - Only works with discrete zenith angles: 0°, 5°, 10°, 15°, ..., 80°, 85°
  - Not recommended for calculations with arbitrary zenith angles from `Z()`

**Usage:**
```r
# Use polynomial implementation (default)
Sys.setenv(NET_FLUX_FUNCTION_TYPE = "polynomial")

# Use lookup table v2
Sys.setenv(NET_FLUX_FUNCTION_TYPE = "lookup_v2")
```

### NET_FLUX_FUNCTION_SHOW_WARNINGS

Controls whether to display warnings about potential calculation errors.

**Options:**

- `TRUE` (default) - Show warnings when polynomial calculations may have notable error margin
- `FALSE` - Suppress warnings

Warnings are shown when:

- Optical depth tau > 5
- Zenith angle z >= 80°

**Usage:**

```r
# Show warnings (default)
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = TRUE)

# Suppress warnings
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)
```

## Development

### Running Tests

To run the package tests during development:
```r
# Install devtools if you haven't already
install.packages("devtools")

# Option 1: Run tests only (faster for quick testing)
devtools::test()

# Option 2: Run a full package check (includes tests, documentation, examples)
devtools::check()
```

**`devtools::test()`** is recommended for quick iteration during development - it loads your package and runs all tests without requiring installation.

**`devtools::check()`** performs a comprehensive package check including tests, documentation, examples, and package structure validation - use this before submitting to CRAN or creating releases.

### Generating Documentation

#### Generating man/ Files

The `man/` directory contains R documentation files (.Rd) generated from roxygen2 comments in the source code. To regenerate these files after modifying function documentation:

```r
# Install roxygen2
install.packages("roxygen2")

# Generate documentation
roxygen2::roxygenize()
```

This updates all `.Rd` files in the `man/` directory based on the `#'` roxygen2 comments in R source files.

#### Generating PDF Reference Manual

The package reference manual is a comprehensive PDF containing documentation for all exported functions.

For Apple Silicon Macs (M1/M2/M3):
```bash
# Build the Docker image (first time only, from repository root)
docker build --platform linux/amd64 -t marsrad-manual -f Dockerfile-manual .

# Generate the PDF
docker run --rm --platform linux/amd64 -v $(pwd):/output marsrad-manual
```

For Intel Macs and Linux:
```bash
# Build the Docker image (first time only, from repository root)
docker build -t marsrad-manual -f Dockerfile-manual .

# Generate the PDF
docker run --rm -v $(pwd):/output marsrad-manual
```

The `marsrad-manual.pdf` PDF will be created in the repository's root directory.

## References

Based on the following excellent work:

- [Appelbaum, Joseph & Flood, Dennis. (1989). Solar Radiation on Mars. NASA TM-102299.](https://ntrs.nasa.gov/citations/19890018252) Detailed information on solar radiation characteristics on Mars are necessary for effective design of future planned solar energy systems operating on the surface of Mars. In this paper we present a procedure and solar radiation related data from which the diurnally, hourly and daily variation of the global, direct beam and diffuse insolation on Mars are calculated. The radiation data are based on measured optical depth of the Martian atmosphere derived from images taken of the sun with a special diode on the Viking cameras; and computation based on multiple wavelength and multiple scattering of the solar radiation.

- [Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars: Update 1990. NASA TM-103623.](https://ntrs.nasa.gov/citations/19910005804) Detailed information on solar radiation characteristics on Mars are necessary for effective design of future planned solar energy systems operating on the surface of Mars. The authors present a procedure and solar radiation related data from which the diurnally and daily variation of the global, direct beam and diffuse insolation on Mars are calculated. The radiation data are based on measured optical depth of the Martian atmosphere derived from images taken of the Sun with a special diode on the Viking Lander cameras and computation based on multiple wavelength and multiple scattering of the solar radiation. This work is an update to NASA-TM-102299 and includes a refinement of the solar radiation model.

- [Appelbaum, Joseph & Flood, Dennis. (1991). Solar radiation on Mars: Update 1991. NASA TM-105216.](https://ntrs.nasa.gov/citations/19910023732) Detailed information on solar radiation characteristics on Mars are necessary for effective design of future planned solar energy systems operating on the surface of Mars. In this paper we present a procedure and solar radiation related data from which the daily variation of the global, direct beam, and diffuse insolation on Mars are calculated. Given the optical depth of the Mars atmosphere, the global radiation is calculated from the normalized net flux function based on multiple wavelength and multiple scattering of the solar radiation. The direct beam was derived from the optical depth using Beer's law, and the diffuse component was obtained from the difference of the global and the direct beam radiation. The optical depths of the Mars atmosphere were derived from images taken of the Sun with a special diode on the cameras used on the two Viking Landers.

- [Appelbaum, Joseph & Sherman, Irving & Landis, Geoffrey. (1993). Solar radiation on Mars: Stationary photovoltaic array. NASA TM-106321.](https://ntrs.nasa.gov/citations/19940010257) Solar energy is likely to be an important power source for surface-based operation on Mars. Photovoltaic cells offer many advantages. In this article we have presented analytical expressions and solar radiation data for stationary flat surfaces (horizontal and inclined) as a function of latitude, season and atmospheric dust load (optical depth). The diffuse component of the solar radiation on Mars can be significant, thus greatly affecting the optimal inclination angle of the photovoltaic surface.

- [Appelbaum, Joseph & Flood, Dennis & Norambuena, Marcos. (1994). Solar radiation on Mars: Tracking photovoltaic array. NASA TM-106700.](https://ntrs.nasa.gov/citations/19950004977) A photovoltaic power source for surface-based operation on Mars can offer many advantages. Detailed information on solar radiation characteristics on Mars and the insolation on various types of collector surfaces are necessary for effective design of future planned photovoltaic systems. In this article we have presented analytical expressions for solar radiation calculation and solar radiation data for single axis (of various types) and two axis tracking surfaces and compared the insulation to horizontal and inclined surfaces. For clear skies (low atmospheric dust load) tracking surfaces resulted in higher insolation than stationary surfaces, whereas for highly dusty atmospheres, the difference is small. The insolation on the different types of stationary and tracking surfaces depend on latitude, season and optical depth of the atmosphere, and the duration of system operation. These insolations have to be compared for each mission.

# Beam daily insolation on Mars horizontal surface  [Wh/m2-day].
#
# Obtained by integrating Equation 19 (1990), beam hourly insolation on Mars horizontal surface,
# over the period from sunrise to sunset.
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252


#' Beam daily insolation on Mars horizontal surface
#'
#' Calculates the direct beam solar energy received over a full Martian day on a horizontal surface.
#' Obtained by integrating beam irradiance from sunrise to sunset.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param tau Atmospheric optical depth (dimensionless)
#'
#' @return Beam daily insolation [Wh/m²-day]
#' @export
H_bh = function(Ls, phi, tau){
  
  # Hbh is obtained by integrating Ibh over the period from sunrise to sunset.
  Hbh = I_bh(Ls=Ls, phi=phi, tau=tau, Ts_start=0, Ts_end=24)
  
  # Return result.
  return(Hbh)
}

# Beam daily insolation on Mars horizontal surface [Wh/m2-day].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252
#
# Expected results taken from:
#   TABLE V. - HOURLY AND DAILY BEAM INSOLATION ON A HORIZONTAL SURFACE AT MARS SURFACE.
#   TABLE V. Daily insolation on a horizontal surface (Update 1990).

context("Beam daily insolation on a horizontal surface")
source("utils.R")

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

#Test with expected results from TABLE V.
test_that("H_bh: Beam daily insolation on a horizontal surface", {
  # 41 Wh/m2 error tolerance.
  tolerance = 41

  phi = spacecrafts$VL1$latitude

  expected_results = list(
    "69" = c(0.65, 1768), # FIXME: Larger error (40).
    "120" = c(0.40, 2534),
    "153" = c(0.50, 2308),
    "249" = c(1.40, 314),
    "299" = c(3.25, 12))

  index = 1
  for(expected_result in expected_results){

    Ls = strtoi(names(expected_results)[index])
    tau = expected_result[1]
    Hbh_expected = expected_result[2]

    Hbh = H_bh(Ls=Ls, phi=phi, tau=tau)
    expect_equal(Hbh, Hbh_expected, tolerance=tolerance, scale=1)

    index = index + 1
  }
})

test_that("H_bh: Beam daily insolation on a horizontal surface at VL1", {
  # 3% Error tolerance.
  tol = 0.03

  # Test input parameter.
  Ls_seq = seq(0, 355, 5)

  # Run the tests.
  test_daily_insolation_on_horizontal_surface(
    spacecraft = "VL1",
    field = "Hbh",
    tolerance = tol,
    Ls_seq = Ls_seq,
    verbose = FALSE)
})

test_that("H_bh: Beam daily insolation on a horizontal surface at VL2", {
  # 3.2% Error tolerance.
  tol = 0.032

  # Test input parameter.
  Ls_seq = seq(0, 355, 5)

  # Run the tests.
  test_daily_insolation_on_horizontal_surface(
    spacecraft = "VL2",
    field = "Hbh",
    tolerance = tol,
    Ls_seq = Ls_seq,
    verbose = FALSE)
})

# Spacecraft properties.
spacecrafts = list(
  "VL1" = list(
    "latitude" = 22.3,
    "longitude" = -49.97,
    "beta_optimal" = 6.5
  ),
  "VL2" = list(
    "latitude" = 47.7,
    "longitude" = 134.29,
    "beta_optimal" = 22
  )
)

# Expected data
expected_data = list(
  "VL1" = list(
    "tau" = read.csv("data/discretized_tau_at_vl1_fig_3_1991_update.csv"),
    "insolation" =
      list(
        "horizontal" = read.csv("data/daily_insolation_on_horizontal_surface_at_vl1_table_v_update_1990.csv"),
        "beta_equals_phi" = read.csv("data/insolation_inclined_beta_eq_phi_vl1_tbl_ii_1993.csv"),
        "beta_optimal" = read.csv("data/insolation_optimal_incl_angle_vl1_tbl_iii_1993.csv")
      )
  ),
  "VL2" = list(
    "tau" = read.csv("data/discretized_tau_at_vl2_fig_3_1991_update.csv"),
    "insolation" =
      list(
        "horizontal" = read.csv("data/daily_insolation_on_horizontal_surface_at_vl2_table_v_update_1990.csv"),
        "beta_equals_phi" = read.csv("data/insolation_inclined_beta_eq_phi_vl2_tbl_ii_1993.csv"),
        "beta_optimal" = read.csv("data/insolation_optimal_incl_angle_vl2_tbl_iii_1993.csv")
      )
  )
)

#' A convenience function to test daily insolation on horizontal surface.
#' 
#' @param spacecraft The spacecraft data to test against: VL1 or VL2. 
#' @param field The insolation values to test: Hbh, Hdh, or Hh.
#' @param tolerance
#' @param Ls_seq 
#' @param verbose
#'
#' @return
test_daily_insolation_on_horizontal_surface = function(spacecraft, field, tolerance, Ls_seq, al=NULL, verbose=FALSE){
  
  # Spacecraft location.
  phi = spacecrafts[[spacecraft]][["latitude"]]
  longitude = spacecrafts[[spacecraft]][["longitude"]]
  
  # Discretized ground truth optical depth measured by the spacecraft.
  measured_taus = expected_data[[spacecraft]][["tau"]]

  # Expected insolation as per the literature.
  expected_insolations = expected_data[[spacecraft]][["insolation"]][["horizontal"]]
  
  for(Ls in Ls_seq){
    # Measured tau.
    tau_measured = measured_taus[measured_taus$Ls == Ls, "tau"]
    
    # Expected insolation.
    val_expected = expected_insolations[expected_insolations$Ls == Ls, field]
    
    # Calculated insolation.
    val_calculated = ifelse(field=="Hbh", H_bh(Ls=Ls, phi=phi, tau=tau_measured),
                                               ifelse(field=="Hdh", H_dh(Ls=Ls, phi=phi, longitude=longitude, tau=tau_measured, al=al),
                                                      ifelse(field=="Hh", H_h(Ls=Ls, phi=phi, longitude=longitude, tau=tau_measured, al=al),
                                                             stop("Invalid field."))))
    
    # Round the calculated value to match floating point numbers in expected value.
    val_calculated = round(val_calculated, 2)
    
    if(isTRUE(verbose)){
      print(paste("Ls:", Ls,
                  " tau:", tau_measured,
                  " ", field, "_calculated:", val_calculated,
                  " ", field, "_expected:", val_expected,
                  " diff:", (val_calculated-val_expected)), sep="")     
    }

    
    # Assert equality.
    expect_equal(val_calculated, val_expected, tolerance=tolerance*val_expected, scale=1)
  }
}


#' Title
#'
#' @param spacecraft The spacecraft data to test against: VL1 or VL2.
#' @param field The insolation values to test: Hbi, Hdi, Hali, or Hi. 
#' @param tolerance 
#' @param Ls_seq 
#' @param beta_equals_phi Test against literature data in which beta = phi. Else use optimal beta angle.
#' @param verbose 
#'
#' @return
test_daily_insolation_on_inclined_surface = function(spacecraft, field, tolerance, Ls_seq, beta_equals_phi=FALSE, verbose=FALSE){
  
  # Spacecraft location.
  phi = spacecrafts[[spacecraft]][["latitude"]]
  longitude = spacecrafts[[spacecraft]][["longitude"]]
  
  # Discretized ground truth optical depth measured by the spacecraft.
  measured_taus = expected_data[[spacecraft]][["tau"]]
  
  expected_insolations = NULL
  if (isTRUE(beta_equals_phi)){
    expected_insolations = expected_data[[spacecraft]][["insolation"]][["beta_equals_phi"]]
  }else{
    expected_insolations = expected_data[[spacecraft]][["insolation"]][["beta_optimal"]]
  }
  
  # Spacecraft orientation.
  gamma_c = 0
  
  # Spacecraft surface inclination angle.
  beta = ifelse(isTRUE(beta_equals_phi),
                spacecrafts[[spacecraft]][["latitude"]],
                spacecrafts[[spacecraft]][["beta_optimal"]])
  
  
  for(Ls in Ls_seq){
    # Measured tau.
    tau_measured = measured_taus[measured_taus$Ls == Ls, "tau"]
    
    # Expected insolation.
    val_expected = expected_insolations[expected_insolations$Ls == Ls, field]
    
    # Calculated insolation.
    val_calculated = ifelse(field=="Hb", H_bi(Ls=Ls, phi=phi, tau=tau_measured, beta=beta, gamma_c=gamma_c),
                            ifelse(field=="Hd", H_di(Ls=Ls, phi=phi, longitude=longitude, tau=tau_measured, beta=beta, gamma_c=gamma_c),
                                   ifelse(field=="Hal", H_ali(Ls=Ls, phi=phi, longitude=longitude, tau=tau_measured, beta=beta, gamma_c=gamma_c),
                                          ifelse(field=="H", H_i(Ls=Ls, phi=phi, longitude=longitude, tau=tau_measured, beta=beta, gamma_c=gamma_c),
                                                 stop("Invalid field.")))))
    
    # Round the calculated valuNULLe to match floating point numbers in expected value.
    val_calculated = round(val_calculated, 1)
    
    if(isTRUE(verbose)){
      print(paste("Ls:", Ls,
                  " tau:", tau_measured,
                  " ", field, "_calculated:", val_calculated,
                  " ", field, "_expected:", val_expected,
                  " diff:", (val_calculated-val_expected)), sep="")     
    }
    
    # Assert equality.
    expect_equal(val_calculated, val_expected, tolerance=tolerance*val_expected, scale=1)
  }
}

# FIXME: Error check that albedo value is given and not calculated when using look up table for net_flux.

# Equation 17 (1990): Global irradiance on Mars horizontal surface [W/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252


#' Global irradiance on Mars horizontal surface
#'
#' Calculates the total solar irradiance (direct beam + diffuse + albedo) incident on a
#' horizontal surface on Mars. Implements Equation 17 from Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param Ts Solar time [h]
#' @param z Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts
#' @param tau Atmospheric optical depth (dimensionless)
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#'
#' @return Global irradiance [W/m²]
#' @export
G_h = function(Ls, phi, longitude, Ts=NULL, z=Z(Ls=Ls, phi=phi, Ts=Ts), tau, al=albedo(latitude=phi, longitude=longitude, tau=tau)){
  
  if(!is_irradiated(Ls=Ls, phi=phi, Ts=Ts, z=z)){
    return(0)
    
  }else{
    # Get the normalized net flux value.
    net_flux = f(z=z, tau=tau, al=al)
    
    # Calculate Gh.
    Gh = G_ob(Ls) * cos(z*pi/180) * (net_flux / (1-al))
    
    # Return the result.
    return(Gh)
  }
}

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

test_that("G_h: Global irradiance on Mars horizontal surface with different normalized net flux functions.", {
  # Test parameters.
  Ls = 122    # Areocentric longitude.
  phi = 22.3  # Planetaryz latitude.
  al = 0.1    # Albedo.
  
  # Solar zenith angle.
  z_seq = c(
    seq(0, 80, 10),
    85)
  
  # Optical depth.
  tau_seq = c(
    seq(0.1, 2.0, 0.1),
    seq(2.5, 3, 0.5),
    5, 6)
  
  for(z in z_seq){
    for(tau in tau_seq){
      
      # Calculate global irradiances for different normalized net flux function types
      Sys.setenv(NET_FLUX_FUNCTION_TYPE = "lookup_v1")
      Gh_lookup_1 = G_h(Ls=Ls, phi=phi, z=z, tau=tau, al=al)
      
      Sys.setenv(NET_FLUX_FUNCTION_TYPE = "lookup_v2")
      Gh_lookup_2 = G_h(Ls=Ls, phi=phi, z=z, tau=tau, al=al)
      
      # Test assert equality.
      expect_equal(Gh_lookup_1, Gh_lookup_2, tolerance=4, scale=1)
      
      # Test against polynomial approach to obtaining the normalized net flux.
      Sys.setenv(NET_FLUX_FUNCTION_TYPE = "polynomial")
      Gh_analytical = G_h(Ls=Ls, phi=phi, z=z, tau=tau, al=al)
      
      # For equality test against analytical approach, set the tolerance based on tau value.
      tolerance = 30
      
      if(tau == 6){
        tolerance = 250
        
      }else if(tau == 5){
        tolerance = 140
      }
      
      expect_equal(Gh_analytical, Gh_lookup_1, tolerance=tolerance, scale=1)
      expect_equal(Gh_analytical, Gh_lookup_2, tolerance=tolerance, scale=1)
    }
  }
})

test_that("G_h: Non-irradiated cases return zero", {
  # Test parameters
  Ls = 90
  phi = 22.48
  longitude = -48
  tau = 0.5

  # Test nighttime (not irradiated) - should return 0
  result_night = G_h(Ls = Ls, phi = phi, longitude = longitude, Ts = 0, tau = tau)
  expect_equal(result_night, 0)

  # Test with sun below horizon (z >= 90)
  result_below = G_h(Ls = Ls, phi = phi, longitude = longitude, z = 95, tau = tau)
  expect_equal(result_below, 0)
})

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

test_that("G_i: Results identical with and without explicit z parameter", {
  # Test parameters
  Ls = 120
  phi = 22.3
  longitude = -48
  Ts = 12
  tau = 0.4
  al = 0.1
  beta = 30
  gamma_c = 0

  # Calculate with z explicitly provided
  z = Z(Ls = Ls, phi = phi, Ts = Ts)
  result_with_z = G_i(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
                      z = z, tau = tau, al = al, beta = beta, gamma_c = gamma_c)

  # Calculate without z (let it be calculated from default)
  result_without_z = G_i(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
                         tau = tau, al = al, beta = beta, gamma_c = gamma_c)

  # Results should be identical
  expect_equal(result_with_z, result_without_z)
})

test_that("G_bi: Results identical with and without explicit z parameter", {
  # Test parameters
  Ls = 90
  phi = 22.48
  Ts = 12
  tau = 0.5
  beta = 25
  gamma_c = 0

  # Calculate with z explicitly provided
  z = Z(Ls = Ls, phi = phi, Ts = Ts)
  result_with_z = G_bi(Ls = Ls, phi = phi, Ts = Ts, z = z,
                       tau = tau, beta = beta, gamma_c = gamma_c)

  # Calculate without z
  result_without_z = G_bi(Ls = Ls, phi = phi, Ts = Ts,
                          tau = tau, beta = beta, gamma_c = gamma_c)

  expect_equal(result_with_z, result_without_z)
})

test_that("G_di: Results identical with and without explicit z parameter", {
  # Test parameters
  Ls = 90
  phi = 22.48
  longitude = -48
  Ts = 12
  tau = 0.5
  beta = 25

  # Calculate with z explicitly provided
  z = Z(Ls = Ls, phi = phi, Ts = Ts)
  result_with_z = G_di(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
                       z = z, tau = tau, beta = beta)

  # Calculate without z
  result_without_z = G_di(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
                          tau = tau, beta = beta)

  expect_equal(result_with_z, result_without_z)
})

test_that("G_ali: Results identical with and without explicit z parameter", {
  # Test parameters
  Ls = 90
  phi = 22.48
  longitude = -48
  Ts = 12
  tau = 0.5
  beta = 25

  # Calculate with z explicitly provided
  z = Z(Ls = Ls, phi = phi, Ts = Ts)
  result_with_z = G_ali(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
                        z = z, tau = tau, beta = beta)

  # Calculate without z
  result_without_z = G_ali(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
                           tau = tau, beta = beta)

  expect_equal(result_with_z, result_without_z)
})

test_that("G_h: Results identical with and without explicit z parameter", {
  # Test parameters
  Ls = 90
  phi = 22.48
  longitude = -48
  Ts = 12
  tau = 0.5

  # Calculate with z explicitly provided
  z = Z(Ls = Ls, phi = phi, Ts = Ts)
  result_with_z = G_h(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
                      z = z, tau = tau)

  # Calculate without z
  result_without_z = G_h(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
                         tau = tau)

  expect_equal(result_with_z, result_without_z)
})

test_that("G_dh: Results identical with and without explicit z parameter", {
  # Test parameters
  Ls = 90
  phi = 22.48
  longitude = -48
  Ts = 12
  tau = 0.5

  # Calculate with z explicitly provided
  z = Z(Ls = Ls, phi = phi, Ts = Ts)
  result_with_z = G_dh(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
                       z = z, tau = tau)

  # Calculate without z
  result_without_z = G_dh(Ls = Ls, phi = phi, longitude = longitude, Ts = Ts,
                          tau = tau)

  expect_equal(result_with_z, result_without_z)
})

#' Albedo-reflected daily insolation on Mars inclined surface
#'
#' Calculates the ground-reflected solar energy received over a full Martian day on an inclined
#' surface. Obtained by integrating albedo-reflected irradiance from sunrise to sunset.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param tau Atmospheric optical depth (dimensionless)
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#' @param beta Surface tilt/slope angle from horizontal [deg]
#' @param gamma_c Surface azimuth angle [deg]. Zero facing equator, east negative, west positive (-180 to +180)
#'
#' @return Albedo-reflected daily insolation on inclined surface [Wh/m²-day]
#' @export
H_ali = function(Ls, phi, longitude, tau, al=albedo(latitude=phi, longitude=longitude, tau=tau), beta, gamma_c){
  
  if(gamma_c > 180 || gamma_c < -180){
    stop("Surface azimuth angle gamma_c must between -180 and 180 degress with zero south, east negative, and west positive.")
  }
  
  # H_alii is obtained by integrating I_ali over the period from sunrise to sunset.
  Hali = I_ali(Ls=Ls, phi=phi, longitude=longitude, tau=tau, Ts_start=0, Ts_end=24, al=al, beta=beta, gamma_c=gamma_c)
  
  # Return result.
  return(Hali)
}

context("Albedo daily insolation on an inclined surface")
source("utils.R")

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

test_that("H_ali: Albedo daily insolation on an inclined surface for beta = phi at VL1.", {

  # Tolerance.
  tolerance = 0.03

  Ls_seq = seq(0, 360, 5)

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL1",
    field = "Hal",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = TRUE,
    verbose = FALSE)

})

test_that("H_ali: Albedo daily insolation on an optimal inclined angle beta = 6.5 at VL1.", {
  
  # Tolerance.
  tolerance = 0.08
  
  # Test input parameters.
  Ls_seq = seq(0, 355, 5) # Areocentric longitude.
  
  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL1",
    field = "Hal",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = FALSE,
    verbose = FALSE)
  
})

# FIXME: Large tolerance.
test_that("H_ali: Albedo daily insolation on an inclined surface for beta = phi at VL2.", {

  # Tolerance.
  tolerance = 0.12

  # Test input parameters.
  Ls_seq = seq(0, 360, 5)

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL2",
    field = "Hal",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = TRUE,
    verbose = FALSE)
})

test_that("H_ali: Albedo daily insolation on an optimal inclined angle beta = 22 at VL2.", {

  # Tolerance.
  tolerance = 0.09

  # Test input parameters.
  Ls_seq = seq(0, 355, 5) # Areocentric longitude.

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL2",
    field = "Hal",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = FALSE,
    verbose = FALSE)
})

test_that("H_ali: gamma_c validation", {
  # Test invalid gamma_c values (out of -180 to 180 range)
  expect_error(
    H_ali(Ls = 90, phi = 22.48, longitude = -48, tau = 0.5, beta = 25, gamma_c = 181)
  )

  expect_error(
    H_ali(Ls = 90, phi = 22.48, longitude = -48, tau = 0.5, beta = 25, gamma_c = -181)
  )
})

# FIXME: Update this function so that it figures out if its a polar night or day.

# Equation 3 (1994): Global irradiance on Mars inclined surface [W/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis & Norambuena, Marcos. (1994).
#   Solar radiation on Mars: Tracking photovoltaic array.
#   Journal of Propulsion and Power. 12. 10.2514/3.24044 
#   https://ntrs.nasa.gov/?R=19950004977


#' Global irradiance on Mars inclined surface
#'
#' Calculates the total solar irradiance (direct beam + diffuse + albedo-reflected) incident
#' on an inclined surface on Mars. Implements Equation 3 from Appelbaum, Flood & Norambuena (1994).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param Ts Solar time [h]
#' @param z Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts
#' @param tau Atmospheric optical depth (dimensionless)
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#' @param beta Surface tilt/slope angle from horizontal [deg]
#' @param gamma_c Surface azimuth angle [deg]. Zero facing equator, east negative, west positive (-180 to +180)
#'
#' @return Global irradiance on inclined surface [W/m²]
#' @export
G_i = function(Ls, phi, longitude, Ts, z=Z(Ls=Ls, phi=phi, Ts=Ts), tau, al=albedo(latitude=phi, longitude=longitude, tau=tau), beta, gamma_c){
  
  if(gamma_c > 180 || gamma_c < -180){
    stop("Surface azimuth angle gamma_c must between -180\u00b0 and +180\u00b0 with zero facing the equator, east negative, and west positive.")
  }

  # Use the z parameter (either provided by user or calculated from default).
  # Reuse the same z value for all components to avoid redundant calculations.
  a = G_bi(Ls=Ls, phi=phi, Ts=Ts, z=z, tau=tau, beta=beta, gamma_c=gamma_c)
  b = G_di(Ls=Ls, phi=phi, longitude=longitude, Ts=Ts, z=z, tau=tau, al=al, beta=beta)
  c = G_ali(Ls=Ls, phi=phi, longitude=longitude, Ts=Ts, z=z, tau=tau, al=al, beta=beta)
  
  Gi = a + b + c
  
  return(Gi)
}

# Suppress R CMD check NOTEs about global variables
utils::globalVariables(c(
  "df_albedo",
  "df_netflux_0p1_lookup_v1",
  "df_netflux_0p1_lookup_v2",
  "df_netflux_0p4_lookup_v2",
  "df_netflux_k0_coeffs",
  "df_netflux_k1_coeffs"
))

#' Diffuse irradiance on Mars inclined surface
#'
#' Calculates the diffuse solar irradiance (scattered by atmospheric dust) incident on an
#' inclined surface on Mars. Accounts for the view factor of the sky from the tilted surface.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param Ts Solar time [h]
#' @param z Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts
#' @param tau Atmospheric optical depth (dimensionless)
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#' @param beta Surface tilt/slope angle from horizontal [deg]
#'
#' @return Diffuse irradiance on inclined surface [W/m²]
#' @export
G_di = function(Ls, phi, longitude, Ts, z=Z(Ls=Ls,  phi=phi, Ts=Ts), tau, al=albedo(latitude=phi, longitude=longitude, tau=tau), beta){

  # Pass Ts along with phi to avoid parameter mismatch warnings
  Gdi = G_dh(Ls=Ls, phi=phi, longitude=longitude, Ts=Ts, z=z, tau=tau, al=al) * cos((beta*pi/180) / 2)^2
  
  # If negative then interpret as no direct irradiance hitting the inclined surface.
  # TODO: Check if negative value is actually possible.
  Gdi = ifelse(Gdi < 0, 0, Gdi)
  return(Gdi)
}

# Diffuse hourly insolation on Mars horizontal surface [Wh/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252
#

#' Diffuse insolation on Mars horizontal surface over time period
#'
#' Calculates the diffuse solar energy (scattered by atmospheric dust) received on a horizontal
#' surface between specified start and end times. Computed as difference between global and beam insolation.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param tau Atmospheric optical depth (dimensionless)
#' @param Ts_start Start of integration period [h]
#' @param Ts_end End of integration period [h]
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#'
#' @return Diffuse insolation over specified time period [Wh/m²]
#' @export
I_dh = function(Ls, phi, longitude, tau, Ts_start, Ts_end, al=albedo(latitude=phi, longitude=longitude, tau=tau)){
  
  Ih = I_h(Ls=Ls, phi=phi, longitude=longitude, tau=tau, Ts_start=Ts_start, Ts_end=Ts_end, al=al) - I_bh(Ls=Ls, phi=phi, tau=tau, Ts_start=Ts_start, Ts_end=Ts_end)
  return(Ih)
}

# Diffuse hourly insolation on Mars horizontal surface [Wh/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252
#
# Expected results taken from:
#   TABLE VI. - HOURLY AND DAILY DIFFUSE INSOLATION ON A HORIZONTAL SURFACE AT MARS SURFACE.

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

tolerance = 11

phi = 22.3
al = 0.1

expected_results = list(
  # Areocentric Longitude.
  "69" = cbind(
    0.65, # Optical depth tau factor.
    c(168, 160, 152, 135, 103, 58, 10) # Expected hourly I_dh from 12:00 to 19:00.
  ),
  "120" = cbind(
    0.40,
    c(125, 124, 122, 116, 98, 65, 13)
  ),
  "153" = cbind(
    0.50,
    c(163, 161, 155, 139, 108, 58, 3)
  ),
  "249" = cbind(
    1.40,
    c(238, 220, 178, 112, 43, 2, 0)
  ))
  # FIXME: Larger errors when Ls = 299°.
  # "299" = cbind(
  #   3.25,
  #   c(167, 147, 106, 61, 24, 1, 0)
  # ))

test_that("I_dh.", {
  expected_result_index = 1
  for(expected_result in expected_results){
    Ls = strtoi(names(expected_results[expected_result_index]))
    tau = expected_result[1,1]
    
    hour_index = 1
    for(Ts_start in 12:18){
      
      Idh = I_dh(Ls=Ls, phi=phi, tau=tau, Ts_start=Ts_start, Ts_end=Ts_start+1, al=al)
      Idh_expected = expected_result[hour_index, 2]
      
      expect_equal(Idh, Idh_expected, tolerance=tolerance, scale=1)
      
      hour_index = hour_index + 1
    }
    
    expected_result_index = expected_result_index + 1
  }
})

# Equation 1 and 2: The optical depth tau factor function.
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Landis, Geoffrey & Sherman, I. (1991). Solar radiation on Mars—Update 1991. Solar Energy. 50. 35-51. 10.1016/0038-092X(93)90006-A:
#   https://www.researchgate.net/publication/223850868_Solar_radiation_on_Mars-Update_1991
#
# From Appelbaum, Joseph & Landis, Geoffrey & Sherman, I (1991):
#     The optical depths at different latitudes other than the ones measured at the Viking Landers 
#   VL1 and VL2 are introduced in this article and are based on the assumption that the optical depth 
#   varies spatially and are expected to be greatest in the source region where dust is raised into the 
#   atmosphere. References [4,5] support this assumption. It is therefore assumed that the optical depths 
#   have a Gaussian distribution centered at the times of the global storms at VL1 and VL2. 
#   The relationship of the optical depth with latitudes may be expressed by a low order polynomial ratio. 
#   The optical depth function coefficients correspond to the measured opacities at VL1 and VL2. This 
#   function: τ(ϕ, Ls), was developed for the case of two global dust storms per Martian year; a model 
#   that corresponds to the Viking Lander observation. 
#     The location of the first global storm is well characterized, however there is some uncertainty 
#   about the location where the second 1977 storm started. We have therefore developed two models for 
#   global dust storm opacity, corresponding to latitude -10° and -30° locations for the second storm.
#
# The optical depth tau factor function.
#   phi     - Latitude [deg].
#   Ls      - Areocentric longitude [deg].
#   model   - Which model to apply:
#             1 - Assumes that the two global dust storms originated both at latitude ϕ = -30°
#                 and took place at Ls_1 = 215° (fist storm) and Ls_2 = 295° (second storm).
#             2 - Assumes that the first global dust storms originated at latitude ϕ_1 = -30°
#                 and the second one at latitude ϕ_2 = -10°.
#                 The times of the storm aaare the same as model 1, at Ls_1 = 215° (fist storm)
#                 and Ls_2 = 295° (second storm).

           
#' Atmospheric optical depth on Mars
#'
#' Calculates the optical depth of the Martian atmosphere as a function of latitude and season.
#' The optical depth varies spatially and temporally, with peaks during global dust storms.
#' Implements Equations 1 and 2 from Appelbaum, Landis & Sherman (1991).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param model Dust storm model (1 or 2). Model 1 assumes both storms originated at -30° latitude.
#'   Model 2 assumes storms at -30° (Ls=215°) and -10° (Ls=295°)
#'
#' @return Atmospheric optical depth (dimensionless, minimum 0.5)
#' @export
#'
#' @examples
#' # Calculate optical depth at Viking Lander 1 site during dust storm season
#' tau <- optical_depth(Ls = 215, phi = 22.3, model = 1)
optical_depth = function(Ls, phi, model=1){
  
  if (model == 1){
    
    a = 16787 * ((1 + phi/150) / (1917 + (phi+38.27)^2))
    b = 0.779 * exp(-(Ls-215)^2 / 730) 
    c = exp(-(Ls-295)^2 / 730)
    
    res =  a * (b + c)

    return(max(0.5, res))
    
  }else if (model == 2){
    
    a = 19500 * ((1 + phi/150) / (4000 + (phi+48.1)^2))
    b = exp(-(Ls-215)^2 / 730)
    c = 12700 * ((1 + phi/410) / (2465 + (phi+13.1)^2))
    d = exp(-(Ls-295)^2 / 730)
    
    res =  (a * b) + (c * d) 
    
    return(max(0.5, res))
    
  }else{
    stop("Unsupportd model number. Should be either 1 or 2.")
  }
  
}

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

test_that("optical_depth: Atmospheric optical depth calculation", {
  # Test parameters from documentation example
  Ls = 215    # Areocentric longitude (dust storm season)
  phi = 22.3  # Viking Lander 1 latitude

  # Test model 1
  tau1 = optical_depth(Ls = Ls, phi = phi, model = 1)
  expect_true(is.numeric(tau1))
  expect_true(tau1 >= 0.5)  # Minimum value enforced by function

  # Test model 2
  tau2 = optical_depth(Ls = Ls, phi = phi, model = 2)
  expect_true(is.numeric(tau2))
  expect_true(tau2 >= 0.5)  # Minimum value enforced by function

  # Models should give different results
  expect_false(identical(tau1, tau2))

  # Test across different seasons
  for (Ls_test in seq(0, 360, 30)) {
    result1 = optical_depth(Ls = Ls_test, phi = phi, model = 1)
    result2 = optical_depth(Ls = Ls_test, phi = phi, model = 2)

    expect_true(is.numeric(result1))
    expect_true(is.numeric(result2))
    expect_true(result1 >= 0.5)
    expect_true(result2 >= 0.5)
  }

  # Test across different latitudes
  for (phi_test in seq(-90, 90, 30)) {
    result1 = optical_depth(Ls = Ls, phi = phi_test, model = 1)
    result2 = optical_depth(Ls = Ls, phi = phi_test, model = 2)

    expect_true(is.numeric(result1))
    expect_true(is.numeric(result2))
    expect_true(result1 >= 0.5)
    expect_true(result2 >= 0.5)
  }

  # Test invalid model number
  expect_error(
    optical_depth(Ls = Ls, phi = phi, model = 3),
    "Unsupportd model number"
  )
})

Sys.setenv(NET_FLUX_FUNCTION_TYPE = "polynomial")
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = TRUE)

#' Get the net flux function type that is set in the NET_FLUX_FUNCTION_TYPE environment variable.
#' Can either be 'lookup_v1', 'lookup_v2', or 'polynomial'.
#' If NET_FLUX_FUNCTION_TYPE has not been set then default to 'polynomial'.
#'
#' @return Character string: 'lookup_v1', 'lookup_v2', or 'polynomial'
#' @noRd
get_net_flux_function_type = function(){
  net_flux_function_type = ifelse(Sys.getenv("NET_FLUX_FUNCTION_TYPE") == "", "polynomial", Sys.getenv("NET_FLUX_FUNCTION_TYPE"))
  return(net_flux_function_type)
}


#' Get the value set in the NET_FLUX_FUNCTION_SHOW_WARNINGS environment variable that determines if net flux function calculation
#' warnings should be thrown for polynomial calculations that have notable error margin.
#'
#' @return Logical TRUE or FALSE
#' @noRd
show_net_flux_function_warnings = function(){
  
  net_flux_function_type = ifelse(Sys.getenv("NET_FLUX_FUNCTION_SHOW_WARNINGS") == "", "true", Sys.getenv("NET_FLUX_FUNCTION_SHOW_WARNINGS"))
  
  if(tolower(net_flux_function_type) %in% c("1", "f", "false", "n", "no")){
    return(FALSE)
    
  }else{
    # Default to TRUE if set value is not anything other than the recognized values for false.
    return(TRUE)
  }
}

# The function.
#
#' Available optical depth values in net flux lookup tables
#'
#' Returns the tau values available in the selected lookup table version.
#' Only works when NET_FLUX_FUNCTION_TYPE is set to 'lookup_v1' or 'lookup_v2'.
#'
#' @return Numeric vector of available tau values
#' @noRd
f_lookup_taus = function(){
  
  net_flux_function_type = get_net_flux_function_type()
  
  if(net_flux_function_type == "lookup_v1"){
    as.numeric(rownames(df_netflux_0p1_lookup_v1))
    
  }else if(net_flux_function_type == "lookup_v2"){
    as.numeric(rownames(df_netflux_0p1_lookup_v2))
    
  }else{
    stop("This function is only available when the NET_FLUX_FUNCTION_TYPE environment variable is set to 'lookup_v1' or 'lookup_v2'.")
  }
}

#' Available zenith angles in net flux lookup tables
#'
#' Returns the zenith angle values available in the selected lookup table version.
#' Only works when NET_FLUX_FUNCTION_TYPE is set to 'lookup_v1' or 'lookup_v2'.
#'
#' @return Numeric vector of available zenith angle values [deg]
#' @noRd
f_lookup_Zs = function(){
  
  net_flux_function_type = get_net_flux_function_type()
  
  if(net_flux_function_type == "lookup_v1"){
    as.numeric(gsub("X", "", colnames(df_netflux_0p1_lookup_v1)))
    
  }else if(net_flux_function_type == "lookup_v2"){
    as.numeric(gsub("X", "", colnames(df_netflux_0p1_lookup_v2)))
    
  }else{
    stop("This function is only available when the NET_FLUX_FUNCTION_TYPE environment variable is set to 'lookup_v1' or 'lookup_v2'.")
  }
}

#' Coefficient lookup for normalized net flux polynomial
#'
#' Retrieves polynomial coefficients for the analytical net flux function.
#' Based on Table IV in Appelbaum & Flood (1990) Update 1990.
#'
#' @param i Tau exponent index (0-5)
#' @param j Zenith angle exponent index (0-5)
#' @param k Albedo exponent (0 or 1)
#'
#' @return Polynomial coefficient value
#' @noRd
p = function(i, j, k){
  
  if(k==0){
    coefficient = df_netflux_k0_coeffs[j+1, paste("X", i, sep="")]
  }else if(k==1){
    coefficient = df_netflux_k1_coeffs[j+1, paste("X", i, sep="")]
  }else{
    stop("Unsupported k value.")
  }
  
  return(coefficient)
}

#' The normalized net flux function's lookup table lookup.
#'
#' The net solar flux integrated over the solar spectrum on the Martian
#' surface was calculated by Pollack based on multiple wavelength and
#' multiple scattering of the solar radiation.
#'
#' Source: Table III in Appelbaum, Joseph & Flood, Dennis. (1990).
#'
#' @param z Zenith angle [deg].
#' @param tau Optical depth.
#' @param al Albedo, can only be 0.1.
#'
#' @return Normalized net flux.
#' @noRd
f_lookup_v1 = function(z, tau, al=0.1){
  
  if(al != 0.1){
    stop("The albedo can only be 0.1 when using f_lookup_v1.")
  }
  
  return(
    unlist( # Unlist in case a sequence of Zs are given instead of a single value (i.e. in the case of integrations).
      df_netflux_0p1_lookup_v1[df_netflux_0p1_lookup_v1$tau == toString(tau), paste("X", z, sep="")],
      use.names=FALSE
    )
  )
}

#' The normalized net flux function's lookup table lookup.
#'
#' Source: Solar radiation on Mars: Update 1990.
#'  Table III (a) and (b) for an albedo of 0.1.
#'  Table IV  (a) and (b) for an albedo of 0.4.
#'
#' @param z Zenith angle [deg].
#' @param tau Optical depth.
#' @param al Albedo, can be 0.1 or 0.4.
#'
#' @return Normalized net flux.
#' @noRd
f_lookup_v2 = function(z, tau, al=0.1){
  
  nnff_df = NULL
  
  if(al == 0.1){
    nnff_df = df_netflux_0p1_lookup_v2
    
  }else if(al == 0.4){
    nnff_df = df_netflux_0p4_lookup_v2
    
  }else{
    stop("The albedo can only be 0.1 or 0.4 when using f_lookup_v2.")
  }
  
  return(
    unlist( # Unlist in case a sequence of Zs are given instead of a single value (i.e. in the case of integrations).
      nnff_df[nnff_df$tau == toString(tau), paste("X", z, sep="")],
      use.names=FALSE
    )
  )
}


#' The analytical expression of the normalized net flux function.
#'
#' The mean error is about 0.7 percent for the full range. For zenith angles up to 40° the error is much smaller.
#' The largest error is for zenith angle of 80° and 85° and for tau greater than 5. The maximum error is about 7 percent.
#' At these large angles and opacities, the error has a minor effect on the calculated daily insolations.
#'
#' Source: Equation 20? in Appelbaum, Joseph & Flood, Dennis (1990) - Update 1990.
#'
#' @param z Zenith angle [deg].
#' @param tau Optical depth.
#' @param al Albedo.
#'
#' @return Normalized net flux.
#' @noRd
f_analytical = function(z, tau, al=0.1){
  
  # Check for and warn against parameters that would result in lagest errors (max. 7%).
  if(tau > 5 && isTRUE(show_net_flux_function_warnings())){
    warning(paste("Large error encountered with tau = ", tau, " greater than 5 (maximum error is 7%). ", 
                  "Consider using the lookup_v1 and lookup_v2 table lookup implementation of the normalized net flux function instead of its polynomial expression.",
                  sep="")
    )
  }
  
  if(isTRUE(show_net_flux_function_warnings())){
    # Use ifelse in case this function is being invoked from an integration in which case Z can be a vector instead of a scalar.
    # If Z is a scalar and we use if() then the following issue will occur: "the condition has length > 1 and only the first element will be used."
    warning_msg = ifelse(z >= 80, paste("Possibly large error encountered with z = ", z, "\u00b0 (maximum error is 7% for Z = 80\u00b0 or Z = 85\u00b0). ",
                                        "Consider using the lookup_v1 and lookup_v2 table lookup implementation of the normalized net flux function instead of its polynomial expression.\n",
                                        sep=""), "")
    
    # Handle warning message.
    for(w_msg in warning_msg){
      if(w_msg != ""){
        warning(w_msg)
      }
    }    
  }
  
  psum = 0
  for(i in seq(0,5,1)){
    for(j in seq(0,5,1)){
      for(k in seq(0,1,1)){
        psum = psum + p(i, j, k) * tau^i * (z/100)^j * al^k
      }
    }
  }
  
  return(psum * (1-al))
}



#' Normalized net flux function
#'
#' Calculates the normalized net solar flux on the Martian surface accounting for multiple
#' wavelength and multiple scattering in the atmosphere. Based on Pollack's calculations
#' presented in Appelbaum & Flood (1990). Can use polynomial expression or lookup tables.
#'
#' @param z Sun zenith angle [deg]
#' @param tau Atmospheric optical depth (dimensionless)
#' @param al Surface albedo (dimensionless, ranges from 0.1 to 0.4)
#'
#' @section Environment Variables:
#' The function behavior can be configured via environment variables:
#'
#' \strong{NET_FLUX_FUNCTION_TYPE}: Controls which implementation to use:
#' \itemize{
#'   \item \code{"polynomial"} (default) - Analytical polynomial expression with ~0.7\% mean error.
#'         Maximum error ~7\% at zenith angles 80-85° and tau > 5.
#'   \item \code{"lookup_v1"} - Lookup table from NASA TM-102299. Albedo fixed at 0.1.
#'   \item \code{"lookup_v2"} - Lookup table from NASA TM-103623. Supports albedo 0.1 and 0.4.
#' }
#'
#' Set with: \code{Sys.setenv(NET_FLUX_FUNCTION_TYPE = "polynomial")}
#'
#' \strong{NET_FLUX_FUNCTION_SHOW_WARNINGS}: Controls warning display (TRUE/FALSE, default: TRUE).
#' Warnings are shown when polynomial calculations may have notable error margin (tau > 5 or z >= 80°).
#'
#' Set with: \code{Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = TRUE)}
#'
#' @return Normalized net flux (dimensionless)
#' @export
f = function(z, tau, al=0.1){
  
  net_flux_function_type = get_net_flux_function_type()
  
  if(net_flux_function_type == "polynomial"){
    net_flux = f_analytical(z, tau, al)
    
  }else if(net_flux_function_type == "lookup_v1"){
    net_flux = f_lookup_v1(z, tau, al)
    
  }else if(net_flux_function_type == "lookup_v2"){
    net_flux = f_lookup_v2(z, tau, al)
    
  }else{
    stop("The NET_FLUX_FUNCTION_TYPE environment variable should be set to 'lookup_v1', 'lookup_v2', or 'polynomial'.")
  }
  
  # Check if given Z results in NULL net flux.
  if(is.null(net_flux)){
    stop(paste("Sun zenith angle z = ", z ,"\u00b0 is not available in the net flux lookup table. Consider using the polynomial function instead.", sep=""))
    
  }
  
  # Check if given tau results in NA net flux.
  # Use ifelse in case this function is being invoked from an integration in which case net_flux can be a vector instead of a scalar.
  isNAs = ifelse(is.na(net_flux), TRUE, FALSE)
  for(isNA in isNAs){
    if(isTRUE(isNA)){
      stop(paste("Optical depth tau factor tau = ", tau," is not available in the net flux lookup table. Consider using the polynomial function instead.", sep=""))
    }
  }
  
  return(net_flux) 
}

Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = TRUE)
# TODO: Write a test script.

#' Check if surface is receiving solar irradiance
#'
#' Determines whether a surface at a given location and time is receiving solar irradiance.
#' Accounts for polar night/day conditions, sunrise/sunset times, and sun position below horizon.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param Ts Solar time [h]
#' @param z Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts
#' @param beta Surface tilt/slope angle from horizontal [deg]. Optional, for inclined surfaces
#' @param gamma_c Surface azimuth angle [deg]. Optional, for inclined surfaces
#'
#' @return TRUE if surface is receiving irradiance, FALSE otherwise
#' @export
is_irradiated = function(Ls, phi, Ts, z=Z(Ls, Ts, phi), beta=NULL, gamma_c=NULL){

  if(isTRUE(identical(z, numeric(0)))){
    stop("One of the following is required: i. Sun zenith angle z [deg] or ii. Both latitude phi [deg] and solar time Ts [h].")

  # Handle vector inputs early (e.g., from integration calculations)
  # Skip validation for vectors and check zenith angle at the end
  }else if(length(z) > 1){
    # Vector input - check if any values indicate no irradiation (z >= 90)
    # Return vector of TRUE/FALSE or single TRUE if all are valid
    if(any(z >= 90)){
      return(ifelse(z >= 90, FALSE, TRUE))
    }else{
      return(TRUE)
    }

  # FIXME: This validation check warns users when they provide both z and phi/Ts with inconsistent values.
  # It may not be strictly necessary since the function uses z when provided, but it helps catch user errors.
  # Consider removing if it causes issues or is deemed unnecessary for the API.
  }else if(!is.null(phi) && !is.null(Ts) && z != Z(Ls=Ls, Ts=Ts, phi=phi)){
    if(isTRUE(show_net_flux_function_warnings())){
      message("Sun zenith angle z [deg] has been provided, ignoring given latitude phi [deg] and solar time Ts [h].")
    }
  }else if(is.null(phi) && !is.null(Ts) || !is.null(phi) && is.null(Ts)) {
    if(isTRUE(show_net_flux_function_warnings())){
      message("A latitude phi [deg] or a solar time Ts [h] has been given but not needed because a Sun zenith angle Z [deg] has been given as well.")
    }
  }else if(!is.null(phi) && !is.null(Ts)){
 
    # There is no irradiance during polar nights.
    if(is_polar_night(Ls=Ls, phi=phi)){
      return(FALSE)
      
    }else if(is_polar_day(Ls=Ls, phi=phi)){
      # Constant solar irradiance during polar day.
      # The sun is out all the time.
      return(TRUE)
      
    }else{
      # There is no irradiance if the solar time is before sunrise or after sunset.
      Tsr = sunrise(Ls=Ls, phi=phi, beta=beta, gamma_c=gamma_c, unit=3)
      Tss = sunset(Ls=Ls, phi=phi, beta=beta, gamma_c=gamma_c, unit=3)

      if(Ts < Tsr || Ts > Tss){
        return(FALSE)
        
      }else{
        return(TRUE)
      }
    }
  }


  # Scalar case: Check if sun is below horizon (z >= 90 degrees)
  if(z >= 90){
    return(FALSE)
  }else{
    return(TRUE)
  }
}

# Global daily insolation on Mars inclined surface [Wh/m2-day].
#
# Obtained by integrating I_h_beta, global hourly insolation on Mars inclined surface, 
# over the period from sunrise to sunset.
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis & Norambuena, Marcos. (1994).
#   Solar radiation on Mars: Tracking photovoltaic array.
#   Journal of Propulsion and Power. 12. 10.2514/3.24044 
#   https://ntrs.nasa.gov/?R=19950004977


#' Global daily insolation on Mars inclined surface
#'
#' Calculates the total solar energy received over a full Martian day on an inclined surface.
#' Obtained by integrating global hourly insolation from sunrise to sunset. Based on
#' Appelbaum, Flood & Norambuena (1994).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param tau Atmospheric optical depth (dimensionless)
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#' @param beta Surface tilt/slope angle from horizontal [deg]
#' @param gamma_c Surface azimuth angle [deg]. Zero facing equator, east negative, west positive (-180 to +180)
#'
#' @return Global daily insolation on inclined surface [Wh/m²-day]
#' @export
H_i = function(Ls, phi, longitude, tau, al=albedo(latitude=phi, longitude=longitude, tau=tau), beta, gamma_c){
  
  if(gamma_c > 180 || gamma_c < -180){
    stop("Surface azimuth angle gamma_c must between -180\u00b0 and 180\u00b0 with zero south, east negative, and west positive.")
  }
  
  # Hi is obtained by integrating Ii over the period from sunrise to sunset.
  Hi = I_i(Ls=Ls, phi=phi, longitude=longitude, tau=tau, Ts_start=0, Ts_end=24, al=al, beta=beta, gamma_c=gamma_c)
  
  # Return result.
  return(Hi)
}

# Equation 3 (1994): Global irradiance on Mars inclined surface [W/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis & Norambuena, Marcos. (1994). Solar radiation on Mars: Tracking photovoltaic array. Journal of Propulsion and Power. 12. 10.2514/3.24044 
#   https://ntrs.nasa.gov/?R=19950004977
#
# Expected results taken from:
#   Table 3: Daily insolation in Whr/m2 on a 2 axis tracking surface, horizontal surface (horiz),
#            inclined surface with Beta = phi - delta, and four modes of single axis tracking surfaces
#            at VL1 for clear skies with tau = 0.5.
#
context("Global irradiance on Mars inclined surface")
source("utils.R")

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

test_that("H_i: Global daily insolation on an inclined surface for beta = phi at VL1.", {

  # Tolerance.
  tolerance = 0.03

  Ls_seq = seq(0, 360, 5)

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL1",
    field = "H",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = TRUE,
    verbose = FALSE)
})

test_that("H_i: Global daily insolation on an optimal inclined angle beta = 6.5 at VL1", {

  # Tolerance.
  tolerance = 0.03

  # Test input parameters.
  Ls_seq = seq(0, 355, 5) # Areocentric longitude.

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL1",
    field = "H",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = FALSE,
    verbose = FALSE)

})

test_that("H_i: Global daily insolation on an inclined surface for beta = phi at VL2", {

  # Tolerance.
  tolerance = 0.07

  # Test input parameters.
  Ls_seq = seq(0, 360, 5)

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL2",
    field = "H",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = TRUE,
    verbose = FALSE)
})

test_that("H_i: Global daily insolation on an optimal inclined angle beta = 22 at VL2", {

  # Tolerance.
  tolerance = 0.04

  # Test input parameters.
  Ls_seq = seq(0, 355, 5) # Areocentric longitude.

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL2",
    field = "H",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = FALSE,
    verbose = FALSE)
})

test_that("H_i: gamma_c validation", {
  # Test invalid gamma_c values (out of -180 to 180 range)
  expect_error(
    H_i(Ls = 90, phi = 22.48, longitude = -48, tau = 0.5, beta = 25, gamma_c = 181)
  )

  expect_error(
    H_i(Ls = 90, phi = 22.48, longitude = -48, tau = 0.5, beta = 25, gamma_c = -181)
  )
})

#' Diffuse daily insolation on Mars inclined surface
#'
#' Calculates the diffuse solar energy (scattered by atmospheric dust) received over a full
#' Martian day on an inclined surface. Obtained by integrating diffuse irradiance from sunrise to sunset.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param tau Atmospheric optical depth (dimensionless)
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#' @param beta Surface tilt/slope angle from horizontal [deg]
#' @param gamma_c Surface azimuth angle [deg]. Zero facing equator, east negative, west positive (-180 to +180)
#'
#' @return Diffuse daily insolation on inclined surface [Wh/m²-day]
#' @export
H_di = function(Ls, phi, longitude, tau, al=albedo(latitude=phi, longitude=longitude, tau=tau), beta, gamma_c){
  
  if(gamma_c > 180 || gamma_c < -180){
    stop("Surface azimuth angle gamma_c must between -180 and 180 degress with zero south, east negative, and west positive.")
  }
  
  # Hdi is obtained by integrating Idi over the period from sunrise to sunset.
  Hdi = I_di(Ls=Ls, phi=phi, longitude=longitude, tau=tau, Ts_start=0, Ts_end=24, al=al, beta=beta, gamma_c=gamma_c)
  
  # Return result.
  return(Hdi)
}

context("Diffuse daily insolation on an inclined surface")
source("utils.R")

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

test_that("H_di: Diffuse daily insolation on an inclined surface for beta = phi at VL1.", {

  # Tolerance.
  tolerance = 0.035

  Ls_seq = seq(0, 360, 5)

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL1",
    field = "Hd",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = TRUE,
    verbose = FALSE)
})

# test_that("H_di: Diffuse daily insolation on optimal inclined surface for beta = 6.5° at VL1.", {
#   
#   # Tolerance.
#   tolerance = 0.03
#   
#   # Test input parameters.
#   Ls_seq = seq(0, 355, 5) # Areocentric longitude.
#   
#   # Expect equals all calculations against all expected.
#   test_daily_insolation_on_inclined_surface(
#     spacecraft = "VL1",
#     field = "Hd",
#     tolerance = tolerance,
#     Ls_seq = Ls_seq,
#     beta_equals_phi = FALSE,
#     verbose = FALSE)
# })
# 
# # FIXME: Large tolerance.
# test_that("H_di: Diffuse daily insolation on an inclined surface for beta = phi at VL2.", {
# 
#   # Tolerance.
#   tolerance = 0.12
# 
#   # Test input parameters.
#   Ls_seq = seq(0, 360, 5)
# 
#   # Expect equals all calculations against all expected.
#   test_daily_insolation_on_inclined_surface(
#     spacecraft = "VL2",
#     field = "Hd",
#     tolerance = tolerance,
#     Ls_seq = Ls_seq,
#     beta_equals_phi = TRUE,
#     verbose = FALSE)
# })
# 
# test_that("H_di: Diffuse daily insolation on optimal inclined surface for beta = 22° at VL2.", {
#   
#   # Tolerance.
#   tolerance = 0.06
#   
#   # Test input parameters.
#   Ls_seq = seq(0, 355, 5) # Areocentric longitude.
#   
#   # Expect equals all calculations against all expected.
#   test_daily_insolation_on_inclined_surface(
#     spacecraft = "VL2",
#     field = "Hd",
#     tolerance = tolerance,
#     Ls_seq = Ls_seq,
#     beta_equals_phi = FALSE,
#     verbose = FALSE)
# })

test_that("H_di: gamma_c validation", {
  # Test invalid gamma_c values (out of -180 to 180 range)
  expect_error(
    H_di(Ls = 90, phi = 22.48, longitude = -48, tau = 0.5, beta = 25, gamma_c = 181)
  )

  expect_error(
    H_di(Ls = 90, phi = 22.48, longitude = -48, tau = 0.5, beta = 25, gamma_c = -181)
  )
})

# From (1993): (19) for inclined and (25) for vertical surface.

#' X parameter for inclined surface sunrise/sunset calculations
#'
#' Calculates the X parameter used in determining sunrise and sunset times for inclined surfaces.
#' Implements Equations 19 and 25 from Appelbaum (1993) for inclined and vertical surfaces.
#'
#' @param phi Planetary latitude [rad]
#' @param beta Surface inclination angle [rad]
#' @param gamma_c Surface azimuth angle [rad]
#'
#' @return X parameter (dimensionless)
#' @noRd
x_for_inclined_surface = function(phi, beta, gamma_c){
  
  # FIXME: For vertical surface, i.e. beta = 90. Need gamma_s?!?!
  #        See (25) in (1993).
  a = ifelse(round(beta, 2) == round(90*pi/180, 2), 0, cos(phi) / (sin(gamma_c) * tan(beta)))
  b = sin(phi) / tan(gamma_c)
  
  x = a + b
  
  return(x)
}


# From (1993): (20) for inclined and (26) for vertical surface.

#' Y parameter for inclined surface sunrise/sunset calculations
#'
#' Calculates the Y parameter used in determining sunrise and sunset times for inclined surfaces.
#' Implements Equations 20 and 26 from Appelbaum (1993) for inclined and vertical surfaces.
#'
#' @param phi Planetary latitude [rad]
#' @param beta Surface inclination angle [rad]
#' @param gamma_c Surface azimuth angle [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Y parameter (dimensionless)
#' @noRd
y_for_inclined_surface = function(phi, beta, gamma_c, delta){
  
  # a is zero for a vertical surface, i.e. beta = 90 deg. See (26) from (1993)
  a = ifelse(round(beta, 2) == round(90*pi/180, 2), 0, sin(phi) / (sin(gamma_c) * tan(beta)))
  b = cos(phi) / tan(gamma_c)
  
  y = tan(delta) * (a - b)
  
  return(y)
}

# Equation 5 (1990): Beam irridiance on a horizontal surface at the top of Mars atmosphere [W/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252


#' Beam irradiance on horizontal surface at top of Mars atmosphere
#'
#' Calculates the solar beam irradiance on a horizontal surface at the top of the Martian
#' atmosphere (before atmospheric attenuation). Implements Equation 5 from Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]. Can be NULL if z is provided
#' @param Ts Solar time [h]. Can be NULL if z is provided
#' @param z Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts
#'
#' @return Beam irradiance at top of atmosphere [W/m²]
#' @export
G_obh = function(Ls, phi=NULL, Ts=NULL, z=Z(Ls=Ls, phi=phi, Ts=Ts)){
  #TODO: What happens if phi and Ts are null?
  Gobh = G_ob(Ls) * cos(z*pi/180)
  return(Gobh)
}

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

test_that("G_obh: Beam irradiance on horizontal surface at top of Mars atmosphere", {
  # Test parameters
  Ls = 90     # Areocentric longitude (northern summer solstice)
  phi = 22.48 # Viking 1 latitude
  Ts = 12     # Solar time (noon)

  # Calculate using phi and Ts
  result1 = G_obh(Ls = Ls, phi = phi, Ts = Ts)
  expect_true(is.numeric(result1))
  expect_true(result1 >= 0)

  # Calculate using explicit zenith angle
  z = Z(Ls = Ls, phi = phi, Ts = Ts)
  result2 = G_obh(Ls = Ls, z = z)
  expect_equal(result1, result2)

  # Test at different times of day
  for (Ts_test in c(6, 9, 12, 15, 18)) {
    result = G_obh(Ls = Ls, phi = phi, Ts = Ts_test)
    expect_true(is.numeric(result))
    expect_true(result >= 0)
  }

  # Test at different seasons
  for (Ls_test in seq(0, 360, 90)) {
    result = G_obh(Ls = Ls_test, phi = phi, Ts = Ts)
    expect_true(is.numeric(result))
    expect_true(result >= 0)
  }
})

# Equation 19 (1990): Beam hourly insolation on Mars horizontal surface [Wh/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252
#


#' Beam insolation on Mars horizontal surface over time period
#'
#' Calculates the direct beam solar energy received on a horizontal surface between specified
#' start and end times. Implements Equation 19 from Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param tau Atmospheric optical depth (dimensionless)
#' @param Ts_start Start of integration period [h]
#' @param Ts_end End of integration period [h]
#'
#' @return Beam insolation over specified time period [Wh/m²]
#' @export
I_bh = function(Ls, phi, tau, Ts_start, Ts_end){

  # Step 1: Constrain Ts_start and Ts_end based on sunrise and sunset times.
  
  # Apply solar time range constraint.
  T_range = constrain_solar_time_range(Ls=Ls, phi=phi, Ts_start=Ts_start, Ts_end=Ts_end)

  # No solar irradiance within the contrained time range.
  if(is.null(T_range)){
    return(0)

  }else{
    # Constrain the time range.
    Ts_start = T_range$Ts_start
    Ts_end = T_range$Ts_end
  }
  
  # Step 2: Calculate beam insolation.

  # The integrand for Equation 19 (1990).
  integrand = function(Ts){
    z = Z(Ls=Ls, phi=phi, Ts=Ts)
    x = G_b(Ls=Ls, phi=phi, Ts=Ts, tau=tau) * cos(z*pi/180)
    
    return(x)
  }

  # Equation 19 (1990): Beam hourly insolation on Mars horizontal surface.
  Ibh = integrate(integrand, Ts_start, Ts_end)

  return(Ibh$value)
}

# Equation 19 (1990): Beam hourly insolation on Mars horizontal surface [Wh/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252
#
# Expected results taken from:
#   TABLE V. - HOURLY AND DAILY BEAM INSOLATION ON A HORIZONTAL SURFACE AT MARS SURFACE.

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

test_that("I_bh.", {

  # Tolerance.
  tolerance = 10

  # Expected data.
  expected_results = list(
    # Areocentric Longitude.
    "69" = cbind(
      0.65, # Optical depth tau factor.
      c(252, 230, 186, 128, 67, 20, 3) # Expected hourly I_bh from 12:00 to 19:00.
    ),
    "120" = cbind(
      0.40,
      c(352, 322, 265, 190, 103, 33, 2)
    ),
    "153" = cbind(
      0.50,
      c(345, 310, 244, 163, 77, 15, 0)
    ),
    "249" = cbind(
      1.40,
      c(69, 50, 26, 10, 2, 0, 0)
    ),
    "299" = cbind(
      3.25,
      c(3, 2, 1, 0, 0, 0, 0)
    ))

  # Test input parameter
  phi = 22.3

  expected_result_index = 1
  for(expected_result in expected_results){
    Ls = strtoi(names(expected_results[expected_result_index]))
    tau = expected_result[1,1]

    hour_index = 1
    for(Ts_start in 12:18){

      # Expected output.
      Ibh_expected = expected_result[hour_index, 2]

      # Calculated output.
      Ibh = I_bh(Ls=Ls, phi=phi, tau=tau, Ts_start=Ts_start, Ts_end=Ts_start+1)

      # Test assertion
      expect_equal(Ibh, Ibh_expected, tolerance=tolerance, scale=1)

      hour_index = hour_index + 1
    }

    expected_result_index = expected_result_index + 1
  }
})

# FIXME: Update this function so that it figures out if its a polar night or day.

# Equation 3 (1994): Global irradiance on Mars inclined surface [W/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis & Norambuena, Marcos. (1994).
#   Solar radiation on Mars: Tracking photovoltaic array.
#   Journal of Propulsion and Power. 12. 10.2514/3.24044 
#   https://ntrs.nasa.gov/?R=19950004977


#' Direct beam irradiance on Mars inclined surface
#'
#' Calculates the direct beam solar irradiance incident on an inclined surface on Mars.
#' Accounts for the sun's angle of incidence on the tilted and oriented surface. Based on
#' Appelbaum, Flood & Norambuena (1994).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param Ts Solar time [h]
#' @param z Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts
#' @param tau Atmospheric optical depth (dimensionless)
#' @param beta Surface tilt/slope angle from horizontal [deg]
#' @param gamma_c Surface azimuth angle [deg]. Zero facing equator, east negative, west positive (-180 to +180)
#'
#' @return Direct beam irradiance on inclined surface [W/m²]
#' @export
G_bi = function(Ls, phi, Ts, z=Z(Ls=Ls, phi=phi, Ts=Ts), tau, beta, gamma_c){
  
  if(gamma_c > 180 || gamma_c < -180){
    stop("Surface azimuth angle gamma_c must between -180 and 180 degress with zero facing the equator, east negative, and west positive.")
  }
  
  # Equation 7 (1990): Declination angle [rad].
  delta = declination(Ls)
  
  # Equation 8 (1990): Hour angle [deg].
  # From Appelbaum, Joseph & Flood, Dennis. (1990):
  #   The ratio of Mars to Earth length of day is 24.65/24.
  #   It is convenient, for calculation purposes, to define a Mars hour
  #   by dividing the Martian day into 24 hr. Using the same relationship
  #   between the Mars solar time T and the hour angle as for the Earth.
  omega_deg = 15 * Ts - 180
  
  # Set to omega to exactly zero when omega near zero because sometimes it is essentially zero with values like -2.8421709430404e-14 deg.
  zero = function(x){
    return(
      ifelse(x > -1e-5 && x < 1e-5, 0, x)
    )
  }
  
  # Apply the function that will set omega to zero if it is close to zero.
  omega_deg = sapply(omega_deg, zero)
  
  # Equation 6 (1993): Solar Azimuth Angle [deg]
  i = sin(phi*pi/180) * cos(delta) * cos(omega_deg*pi/180)
  j = cos(phi*pi/180) * sin(delta)
  k = sin(z*pi/180)
  
  # From (32) in (1993): It is solar noon when omega is 0°. This translates to gamma_s = 0°.
  # The following operation will result in a value very close to 1 but not exactly 1 if it is
  # solar noon, i.e. when omega is 0 deg. When op = 1 -> acos(op) = 0, i.e. (32) in (1993).
  op = (i - j) / k
  
  # The following function is to make sure that op is:
  #   - exactly 1 when omega is 0° (or else NaN is produced when calculating gamma_s)
  #   - exactly 0 when omega is 90°.
  adjust = function(x){
    return(
      ifelse(x > -1e-5 && x < 1e-5, 0,
             ifelse(x > 0,
                    ifelse(x > 1 && x < 1+1e-5, 1, x),  # If x is positive.
                    ifelse(x < -1 && x > -1-1e-5, 1, x))) # If x is negative.
    )
  }

  # Apply the function.
  op = sapply(op, adjust)
  
  # Calculate gamma_s.
  gamma_s = acos(op) * 180/pi # [deg]
    
  # Alternatively, Equation 7 (1993): Solar Azimuth Angle [deg]
  # x = sin(phi*pi/180) * cos(z*pi/180) - sin(delta)
  # y = cos(phi*pi/180) * sin(z*pi/180)
  # 
  # gamma_s = acos(x / y) * 180/pi # [deg]

  # Sun Angle of Incidence [rad].
  sun_angle_of_incidence = function(){
    teta = NULL
    
    # (23) in (1993) Vertical surface.
    if(abs(beta) == 90){
      i = cos(gamma_s * pi/180) * cos(gamma_c * pi/180)
      #TODO:  Double check this, why wouldn't the paper use squared instead of multiply the by the same value? 
      #       Check with (24) in (1993). 
      j = sin(gamma_s * pi/180) * sin(gamma_s * pi/180) 
      teta = acos(sin(z * pi/180) * (i + j))# [rad]
      
    }else{
      # (13) in (1993): Inclined surface.
      # (27) in (1993): Surface facing the equator.
      i = cos(beta * pi/180) * cos(z * pi/180)
      j = sin(beta * pi/180) * sin(z * pi/180) * cos((gamma_s - gamma_c) * pi/180) # Does not matter when beta = 0 because it leads to j = 0.
      teta = acos(i + j) # [rad]
    }
    
    return(teta)
  }
  
  # Sun Angle of Incidence [rad] on an inclined surface.
  teta = sun_angle_of_incidence()
 
  # Calculate Gbi.
  Gbi = G_b(Ls=Ls, z=z, tau=tau) * cos(teta)
  
  # For certain teta values we get negative Gbi.
  # This is interpreted as no beam irradiance hitting the inclined surface.
  Gbi = ifelse(Gbi < 0, 0, Gbi)
  return(Gbi)
}

# Global hourly insolation on Mars horizontal surface [Wh/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252


#' Global insolation on Mars horizontal surface over time period
#'
#' Calculates the total solar energy received on a horizontal surface between specified start
#' and end times. Obtained by integrating global irradiance over the time period.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param tau Atmospheric optical depth (dimensionless)
#' @param Ts_start Start of integration period [h]
#' @param Ts_end End of integration period [h]
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#'
#' @return Global insolation over specified time period [Wh/m²]
#' @export
I_h = function(Ls, phi, longitude, tau, Ts_start, Ts_end, al=albedo(latitude=phi, longitude=longitude, tau=tau)){
  
  # Step 1: Constrain Ts_start and Ts_end based on sunrise and sunset times.
  
  # Apply solar time range constraint.
  T_range = constrain_solar_time_range(Ls=Ls, phi=phi, Ts_start=Ts_start, Ts_end=Ts_end)
  
  # No solar irradiance within the contrained time range.
  if(is.null(T_range)){
    return(0)
    
  }else{
    # Constrain the time range.
    Ts_start = T_range$Ts_start
    Ts_end = T_range$Ts_end
  }
  
  # Step 2: Calculate insolation.
  
  # The integrand for Equation 19 (1990).
  integrand = function(Ts){
    Gh = G_h(Ls=Ls, phi=phi, longitude=longitude, Ts=Ts, tau=tau, al=al)
    return(Gh)
  }
  
  # Global hourly insolation on Mars horizontal surface.
  Ih = integrate(integrand, Ts_start, Ts_end)
  
  return(Ih$value)
}

# Global hourly insolation on Mars horizontal surface [Wh/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252
#
# Expected results taken from:
#   TABLE IV. - HOURLY AND DAILY GLOBAL INSOLATION ON A HORIZONTAL SURFACE AT MARS SURFACE.

# Expected hourly I_h for different areocentric longitudes.
# Test for the 1 hour time ranges at 12-13, 13-14, 14-15, 15-16, 16-17, 17-18, and 18-19.

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)


test_that("I_h: Hourly global insolation on a horizontal surface at mars surface for VL1 location.", {
  
  # Test tolerance
  tolerance = 9
  
  expected_results = list(
    "69" = cbind(
      0.65, # Optical depth tau factor.
      c(420, 390, 338, 263, 170, 78, 11)
    ),
    "120" = cbind(
      0.40,  # Optical depth tau factor.
      c(477, 446, 387, 306, 201, 98, 15)
    ),
    "153" = cbind(
      0.50,  # Optical depth tau factor.
      c(508, 471, 399, 302, 185, 73, 3)
    ),
    "249" = cbind(
      1.40,  # Optical depth tau factor.
      c(307, 270, 204, 122, 45, 2, 0)
    ))
  # #FIXME: Larger errors when Ls = 299°.
  # "299" = cbind(
  #   3.25,  # Optical depth tau factor.
  #   c(170, 149, 107, 61, 24, 1, 0)
  # ))
  
  
  # Constant input parameters.
  phi = 22.3
  longitude = -49.97
  al = 0.1
  
  
  expected_result_index = 1
  for(expected_result in expected_results){
    
    Ls = strtoi(names(expected_results[expected_result_index]))
    tau = expected_result[1,1]

    hour_index = 1
    for(Ts_start in 12:18){
      
      Ih = I_h(Ls=Ls, phi=phi, longitude=longitude, tau=tau, Ts_start=Ts_start, Ts_end=Ts_start+1, al=al)

      Ih_expected = expected_result[hour_index, 2]
      
      expect_equal(Ih, Ih_expected, tolerance=tolerance, scale=1)
      
      hour_index = hour_index + 1
    }
    
    expected_result_index = expected_result_index + 1
  }
})

#' Optimal tilt angle for maximum daily insolation
#'
#' Calculates the optimal surface tilt angle (beta) that maximizes daily solar energy collection
#' for a given location and season on Mars. The surface is assumed to face the equator.
#' Based on Equations 40 and 43 from Appelbaum (1993).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param unit Output unit: 1 for radians, 2 for degrees (default: 1)
#'
#' @return Optimal tilt angle [rad] or [deg] depending on unit parameter
#' @export
#'
optimal_angle = function(Ls, phi, unit=1){
  
  if(!unit %in% c(1, 2)){
    stop("Unit option must either be 1 for radians or 2 for degrees.")
  }
  
  # Declination angle [deg].
  delta = declination(Ls, unit=2)

  # Sunrise hour angle [deg].
  omega_sr = sunrise(Ls=Ls, phi=phi, unit=2)
  
  # Optimal angle [rad].
  a1 = cos(omega_sr*pi/180) * sin(phi*pi/180) * cos(delta*pi/180)
  b1 = 1 - (2*(omega_sr*pi/180)/pi)^2
  c1 = cos(phi*pi/180) * sin(delta*pi/180)
  
  a2 = cos(omega_sr*pi/180) * cos(phi*pi/180) * cos(delta*pi/180)
  b2 = 1 + (2*(omega_sr*pi/180)/pi)^2
  c2 = sin(phi*pi/180) * sin(delta*pi/180)
  
  beta = -atan((a1 - (b1 * c1)) / (a2 + (b2 * c2)))
  
  # FIXME: Getting opposite of expected sign when Ls=180.
  # Fix this temporary fix or is it OK?
  beta = ifelse(Ls == 180, -beta, beta)
  
  if(unit == 1){
    return(beta)
    
  }else if(unit == 2){
    return(beta*180/pi)
    
  }else{
    # This should not happen.
    stop("An unknown error has occurred.")
  }  
}

#' Albedo-reflected irradiance on Mars inclined surface
#'
#' Calculates the solar irradiance reflected from the Martian surface (ground-reflected radiation)
#' incident on an inclined surface. Accounts for the view factor of the ground from the tilted surface.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param Ts Solar time [h]
#' @param z Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts
#' @param tau Atmospheric optical depth (dimensionless)
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#' @param beta Surface tilt/slope angle from horizontal [deg]
#'
#' @return Albedo-reflected irradiance on inclined surface [W/m²]
#' @export
G_ali = function(Ls, phi, longitude, Ts, z=Z(Ls=Ls, Ts=Ts, phi=phi), tau, al=albedo(latitude=phi, longitude=longitude, tau=tau), beta){

  # Pass Ts along with phi to avoid parameter mismatch warnings
  Gali = al * G_h(Ls=Ls, phi=phi, longitude=longitude, Ts=Ts, z=z, tau=tau, al=al) * sin((beta*pi/180) / 2)^2
  
  # If negative then interpret as no albedo irradiance hitting the inclined surface.
  # TODO: Check if negative value is actually possible.
  Gali = ifelse(Gali < 0, 0, Gali)
  return(Gali)
}

# FIXME: Refactor to daylight_range without Ts_start and Ts_end.


#' Constrain solar time range to daylight hours
#'
#' Adjusts the specified time range to only include periods when the sun is above the horizon.
#' Accounts for sunrise/sunset times and polar day/night conditions. Returns NULL if no sunlight
#' occurs during the specified period.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param Ts_start Start time to constrain [h]
#' @param Ts_end End time to constrain [h]
#' @param beta Surface tilt/slope angle from horizontal [deg]. Optional, for inclined surfaces
#' @param gamma_c Surface azimuth angle [deg]. Optional, for inclined surfaces
#'
#' @return List with Ts_start and Ts_end constrained to daylight hours, or NULL if no daylight
#' @noRd
constrain_solar_time_range = function(Ls, phi, Ts_start, Ts_end, beta=NULL, gamma_c=NULL){
  
  if(Ts_start >= Ts_end){
    stop("Solar start time cannot be after or equal to the solar end time.")
  }
  
  # If polar night.
  if(is_polar_night(Ls=Ls, phi=phi)){
    # No solar irradiance.
    return(NULL);
  }
  # If polar day.
  else if(is_polar_day(Ls=Ls, phi=phi)){
    # No constraining required: constant solar irradiance during polar day.
    # FIXME: What about for inclined surface?
    #        As in, what if the Sun is just above the horizon on the back of the inclined surface?
    return(
      list(Ts_start=Ts_start,
           Ts_end=Ts_end)
    )
  }
  # If non polar nights and non polar days.
  else{
    # Constrain Ts_start and Ts_end with respect to sunrise and sunset times.
    Tsr = sunrise(Ls=Ls, phi=phi, beta=beta, gamma_c=gamma_c, unit=3)
    Tss = sunset(Ls=Ls, phi=phi, beta=beta, gamma_c=gamma_c, unit=3)
    
    # In case there is no sunrise for the inclined surface.
    # e.g. high northern planetary latitude, oriented northwards.
    if(is.na(Tsr)){
      return(NULL)
    }
    
    # If start time is after the sunset, then there is no solar irradiance.
    else if(Ts_start > Tss){
      return(NULL)
    }
    # If end time is before the sunrise, then there is no solar irradiance.
    else if(Ts_end < Tsr){
      return(NULL)
      
    }else{
      # Be careful to cap the start hour angle to that of the sunrise hour angle.
      #   If you do not do this then you will calculate insolation for a 
      #   negative sunset hour angle, i.e. when the sun is below the horizon.
      if(Ts_start < Tsr){
        # Constrain the given solar time range to times after the sunrise time.
        Ts_start = Tsr
      }
      
      # Be careful to cap the end hour angle to that of the sunset hour angle.
      #   If you do not do this then you will calculate insolation for a 
      #   negative sunset hour angle, i.e. when the sun is below the horizon.
      if(Ts_end > Tss){
        # Constrain the given solar time range to time before to the sunset time.
        Ts_end = Tss
      }
    }
  }
  
  # When applying sunrise and sunset constraint has resulted in the start time being after or equal to the solar end time.
  if(Ts_start >= Ts_end){
    return(NULL)
  }
  
  # Return the contrained time range.
  return(
    list(Ts_start=Ts_start,
         Ts_end=Ts_end)
  )
}

# Mars obliquity of rotation axis [deg].
delta_0 = 24.936

#' Solar declination angle on Mars
#'
#' Calculates the angular position of the Sun at solar noon with respect to the plane of the
#' Martian equator. For Mars: -24.936° <= delta <= 24.936°. The declination is 0° at vernal
#' (Ls=0°) and autumnal equinoxes (Ls=180°), +24.936° at summer solstice (Ls=90°), and -24.936°
#' at winter solstice (Ls=270°). Implements Equation 7 from Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param unit Output unit: 1 for radians, 2 for degrees (default: 1)
#'
#' @return Declination angle [rad] or [deg] depending on unit parameter
#' @export
declination = function(Ls, unit=1){

  if(!unit %in% c(1, 2)){
    stop("Declination angle unit option must either be 1 for radians or 2 for degrees.")
  }
  
  # For Ls = 180, we get delta = 2.95828423403174e-15 instead of delta = 0.
  # This is a hardcoded fix:
  if(Ls == 180){
    return(0)
  }
  
  # Calculate declination angle in radians and degrees.
  delta_rad = asin(sin(delta_0*pi/180) * sin(Ls*pi/180))
  delta_deg = delta_rad * 180/pi
  
  # # Alternative equation for delta.
  # # Source: https://www.giss.nasa.gov/tools/mars24/help/algorithm.html
  # delta_rad = asin(0.42565 * sin(Ls*pi/180)) + 0.25 * sin(Ls*pi/180)
  
  # Return result.
  if(unit == 1){
    return(delta_rad)
    
  }else if(unit == 2){
    return(delta_deg)
    
  }else{
    # This should not happen.
    stop("An unknown error has occurred.")
  }
}

# Equation 12 (1990): Beam insolation on a horizotal surface at the top of Mars atmosphere [Wh/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252
#
# Expected results taken from:
#   TABLE II. - HOURLY AND DAILY BEAM IN SOLATION ON A HORIZONTAL SURFACE AT TOP OF MARS ATMOSPHERE

context("Beam insolation on a horizotal surface at the top of Mars atmosphere")

test_that("I_obh: Integration yields same results as equivalent formulas.",{
  
  # Tolerance.
  tolerance = 1e-10
  
  # Equation 11 in 1989.
  I_obh_eq1 = function(Ls, phi, Ts_start, Ts_end){
    
    # Declination.
    delta = declination(Ls)
    
    integrand = function(omega){
      i = sin(phi*pi/180) * sin(delta) + cos(phi*pi/180) * cos(delta) * cos(omega)
      return(i)
    }
    
    omega_start_rad = (15 * Ts_start - 180)*pi/180
    omega_end_rad = (15 * Ts_end - 180)*pi/180
    
    x =  integrate(integrand, omega_start_rad, omega_end_rad)
    Iobh = 12/pi * G_ob(Ls) * x$value
    
    return(Iobh)
  }
  
  # Equation 12 in 1989.
  I_obh_eq2 = function(Ls, phi, Ts_start, Ts_end){
    
    # Declination.
    delta = declination(Ls)

    # Hour angles [deg].
    omega_start = 15 * Ts_start - 180
    omega_end = 15 * Ts_end - 180

    a = (2 * pi * (omega_end - omega_start)) / 360
    b = sin(phi*pi/180) * sin(delta)
    c = cos(phi*pi/180) * cos(delta) * (sin(omega_end*pi/180) - sin(omega_start*pi/180))

    Iobh = 12/pi * G_ob(Ls) * (a * b + c)
    
    return(Iobh)
  }
  
  # Test.
  # for(Ls in seq(0, 360, 20)){
  #   for(phi in seq(-60, 60, 20)){
  #     for(Ts_start in 12:18){
  for(Ls in 100){
    for(phi in 30){
      for(Ts_start in 13){
        # Calculate insolations.
        Iobh = I_obh(Ls=Ls, phi=phi, Ts_start=Ts_start, Ts_end=Ts_start+1)
        Iobh_eq1 = I_obh_eq1(Ls=Ls, phi=phi, Ts_start=Ts_start, Ts_end=Ts_start+1)
        Iobh_eq2 = I_obh_eq2(Ls=Ls, phi=phi, Ts_start=Ts_start, Ts_end=Ts_start+1)

        # Compare them.
        expect_equal(Iobh_eq1, Iobh_eq2, tolerance=tolerance, scale=1)
      }
    }
  }
})

test_that("I_obh.", {

  # Tolerance
  tolerance = 9

  # Test input parameters
  phi = 22.3

  # Expected hourly I_obh for different areocentric longitudes.
  # Test for the 1 hour time ranges at 12-13, 13-14, 14-15, 15-16, 16-17, 17-18, and 18-19.
  expected_results = list(
    "69" = c(488, 460, 405, 328, 234, 128, 25),
    "120" = c(528, 497, 437, 353, 249, 134, 23),
    "153" = c(572, 536, 467, 368, 247, 113, 7),
    "249" = c(496, 455, 376, 263, 126, 8, 0),
    "299" = c(478, 439, 364, 257, 127, 19, 0))


  # Test.
  expected_result_index = 1
  for(expected_result in expected_results){
    Ls = strtoi(names(expected_results[expected_result_index]))

    hour_index = 1
    for(Ts_start in 12:18){
      Iobh = I_obh(Ls=Ls, phi=phi, Ts_start=Ts_start, Ts_end=Ts_start+1)
      Iobh_expected = expected_result[hour_index]

      expect_equal(Iobh, Iobh_expected, tolerance=tolerance, scale=1)

      hour_index = hour_index + 1
    }

    expected_result_index = expected_result_index + 1
  }
})

context("Declination")

test_that("Declination (delta).", {
  tolerance = 0
  
  # Ls, Delta
  # TODO: Include more test cases.
  expected_results = list(
    "0" = 0,
    "180" = 0,
    "90" = 24.936,
    "270" = -24.936)
  
  index = 1
  for(delta_expected in expected_results){
    # Ls test parameter.
    Ls = strtoi(names(expected_results[index]))
    
    # Calculate declination [deg].
    delta_calculated = declination(Ls, 2) 
    
    # Test assert equals
    expect_equal(delta_calculated, delta_expected, tolerance=tolerance, scale=1)
    
    index = index + 1
  }
})

# Equation 13 (1990): Daily beam insolation on a horizontal surface at top of Mars atmosphere [Wh/m2-day].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252
#
# Expected results taken from:
#   TABLE II. - HOURLY AND DAILY BEAM INSOLATION ON A HORIZONTAL SURFACE AT TOP OF MARS ATMOSPHERE
context("Daily beam insolation on a horizontal surface at top of Mars atmosphere")


test_that("H_obh: Integration yields same results as equivalent formula.",{
  
  # Tolerance.
  tolerance = 1e-10
  
  # Formula that is equivalent to integration I_obh from sunrise to sunset.
  H_obh_eq = function(Ls, phi){
    
    # Equation 7 (1990): Declination angle [rad].
    delta = declination(Ls)
    
    # Equation 9 (1990): The sunset hour angle [rad].
    omega_ss = sunset(Ls=Ls, phi=phi, unit=1)
    
    # Equation 13 (1990): Daily beam insolation on a horizontal surfce at top of Mars atmosphere [Wh/m2-day].
    a = (24/pi) * G_ob(Ls)
    b = 2 * pi * (omega_ss*180/pi) / 360
    c = sin(phi*pi/180) * sin(delta)
    d = cos(phi*pi/180) * cos(delta) * sin(omega_ss)
    
    Hobh = a * (b * c + d)
    
    # Return result.
    return(Hobh)    
  }
  
  # Test.
  for(Ls in seq(0, 360, 10)){
    for(phi in seq(-60, 60, 10)){
      # Calculate insolations.
      Hobh = H_obh(Ls=Ls, phi=phi)
      Hobh_eq = H_obh_eq(Ls=Ls, phi=phi)
      
      # Compare them.
      expect_equal(Hobh, Hobh_eq, tolerance=tolerance, scale=1)
    }
  }
})

# Test with expected results from TABLE II.
test_that("Equation 13 (1990): H_obh.", {

  # Tolerance.
  tolerance = 6

  # Constant test input parameters.
  phi = 22.3

  # Expected results: Ls and H_obh.
  expected_results = list(
    "69" = 4136,
    "120" = 4442,
    "153" = 4620,
    "249" = 3449,
    "299" = 3350)

  index = 1
  for(Hobh_expected in expected_results){
    Ls = strtoi(names(expected_results)[index])
    Hobh = H_obh(Ls, phi)

    expect_equal(Hobh, Hobh_expected, tolerance=tolerance, scale=1)

    index = index + 1
  }
})

# Constants.
Ls_P = 248      # Periphelion.
e = 0.093377    # Mars orbit eccentricity.
Mb = 590        # Mean beam irradiance at the top of the Martian atmosphere.

#' Beam irradiance at top of Martian atmosphere
#'
#' Calculates the solar beam irradiance at the top of the Martian atmosphere (before any
#' atmospheric effects) as a function of Mars' orbital position. Accounts for Mars' elliptical
#' orbit which causes seasonal variation in solar intensity. Implements Equation 4 from
#' Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#'
#' @return Beam irradiance at top of atmosphere [W/m²]
#' @export
G_ob = function(Ls){
  
  Gob = Mb * (1 + e*cos((Ls-Ls_P)*pi/180))^2 / (1-e^2)^2
  return(Gob)
}

# Source: Table I - The albedo function.
#   Appelbaum, Joseph & Landis, Geoffrey & Sherman, I. (1991).
#   Solar radiation on Mars — Update 1991. Solar Energy. 50. 35-51. 10.1016/0038-092X(93)90006-A:
#   https://www.researchgate.net/publication/223850868_Solar_radiation_on_Mars-Update_1991


#' The albedo function.
#' 
#' Calculate the albedo value given geographical location and tau factor.
#' Source: Appelbaum, Joseph & Landis, Geoffrey & Sherman, I. (1991).  Solar radiation on Mars — Update 1991.
#'
#' @param latitude Planetary latitude [deg], from -90° to +90°
#' @param longitude Planetary longitude [deg], from -180° to 180°
#' @param tau Atmospheric optical depth (dimensionless)
#' @param coordinates_rounding Logical, whether to round coordinates to nearest grid point
#'
#' @return Surface albedo value (dimensionless, 0-1)
#' @export
albedo = function(latitude, longitude, tau, coordinates_rounding=TRUE){

  # Check if within boundary.  
  if(longitude < -180 || longitude > 180){
    stop("Longitude must be a value between -180\u00b0 and 180\u00b0.")
    
  }else if(latitude < -90 || latitude > 90){
    stop("Latitude (phi) must be a value between -90\u00b0 and 90\u00b0.")
  }
  
  # Round to a multiple of 10 because the lookup table only has mutiple of tens for longitude and latitude.
  if(isTRUE(coordinates_rounding)){
    longitude = round(longitude, digits=-1)
    latitude = round(latitude, digits=-1) 
  }
  
  # Check if latitude and longitude are multiples of 10 (in case coordinates_rounding=FALSE)
  if(longitude %% 10 != 0 || latitude %% 10 != 0){
    stop("Coordinate values must be multiples of 10.")
  }
  
  # Turn latitude numeric value into column label string value.
  if(latitude == 0){
    latitude = toString(latitude)
    
  }else if(latitude > 0){
    latitude = paste("p", latitude, sep="")
    
  }else if(latitude < 0){
    latitude = paste("m", abs(latitude), sep="")
  }
  
  # Get the albedo from the albedo lookup table.
  al = df_albedo[df_albedo$Longitude == longitude, latitude]
  
  # For periods with dust storms, introduce the following albedo function from (3) in (1991 Update):
  if(tau > 1){
    al = max(c(al, min(c(0.18*tau, 0.4))))
  }
  
  return(al)
}

context("Albedo")

# From Table I in (1991 Update).
test_that("Mars surface albedo.", {
  
  # Test normal inputs.
  expect_equal(albedo(latitude=40, longitude=-70, tau=0.5), 0.265)
  expect_equal(albedo(latitude=-10, longitude=120, tau=0.5), 0.185)
  
  # Test boundaries.
  expect_equal(albedo(latitude=90, longitude=180, tau=0.5), 0.4)
  expect_equal(albedo(latitude=-90, longitude=-180, tau=0.5), 0.375)
  expect_equal(albedo(latitude=0, longitude=0, tau=0.5), 0.2)
  
  # Test rounding to multiple of 10.
  expect_equal(albedo(latitude=22.3, longitude=-49.97, tau=0.5), 0.225) # VL1.
  expect_equal(albedo(latitude=47.7, longitude=134.29, tau=0.5), 0.23) # VL2.
  expect_equal(albedo(latitude=-22, longitude=-166, tau=0.5), 0.225)
  expect_equal(albedo(latitude=85.1, longitude=175.3423, tau=0.5), 0.4)
  
  # Test albedo max out at 0.4 for large tau.
  expect_equal(albedo(latitude=0, longitude=0, tau=5), 0.4)
  
  # Test out of bound errors.
  expect_error(albedo(latitude=91, longitude=-166, tau=0.5))
  expect_error(albedo(latitude=89, longitude=181, tau=0.5))
  expect_error(albedo(latitude=-91, longitude=-166, tau=0.5))
  expect_error(albedo(latitude=89, longitude=-181, tau=0.5))
})

#' Check if location is experiencing polar night
#'
#' Determines whether a given location on Mars is experiencing polar night (24-hour darkness)
#' for the specified season. During polar night, the sun remains below the horizon continuously.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#'
#' @return TRUE if experiencing polar night, FALSE otherwise
#' @export
is_polar_night = function(Ls, phi){
  
  # Equation 7 (1990): Declination angle [rad].
  delta = declination(Ls)
  
  # Equations 16 (Update 1991): Figure out if it is polar night or polar day.
  #   Polar night (polar_flag < -1), no solar irradiance.
  #   Polar day (polar_flag > 1), constant solar irradiance. 
  polar_flag = -tan(delta) * tan(phi*pi/180)
  
  # If polar night.
  if(polar_flag < -1){
    # No solar irradiance.
    return(TRUE)
    
  }else{
    return(FALSE)
  }
}

# Equation 13 (1990): Daily beam insolation on a horizontal surfce at top of Mars atmosphere [Wh/m2-day].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252


#' Daily beam insolation at top of Mars atmosphere
#'
#' Calculates the solar beam energy over a full Martian day on a horizontal surface at the top
#' of the Martian atmosphere (before atmospheric attenuation). Implements Equation 13 from
#' Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#'
#' @return Daily beam insolation at top of atmosphere [Wh/m²-day]
#' @export
H_obh = function(Ls, phi){
  Hobh = I_obh(Ls=Ls, phi=phi, Ts_start=0, Ts_end=24)
  return(Hobh)
}

#' Albedo lookup table
#'
#' Lookup table for Mars surface albedo based on latitude and longitude.
#' Based on NASA Technical Memorandum data.
#'
#' @format A data frame with albedo values indexed by latitude and longitude
#' @keywords internal
"df_albedo"

#' Net flux lookup table (albedo 0.1, version 1)
#'
#' Lookup table for normalized net flux function with albedo fixed at 0.1.
#' From NASA TM-102299 (Appelbaum & Flood, 1989).
#'
#' @format A data frame with net flux values indexed by zenith angle and optical depth
#' @keywords internal
"df_netflux_0p1_lookup_v1"

#' Net flux lookup table (albedo 0.1, version 2)
#'
#' Lookup table for normalized net flux function with albedo 0.1.
#' From NASA TM-103623 (Appelbaum & Flood, 1990).
#'
#' @format A data frame with net flux values indexed by zenith angle and optical depth
#' @keywords internal
"df_netflux_0p1_lookup_v2"

#' Net flux lookup table (albedo 0.4, version 2)
#'
#' Lookup table for normalized net flux function with albedo 0.4.
#' From NASA TM-103623 (Appelbaum & Flood, 1990).
#'
#' @format A data frame with net flux values indexed by zenith angle and optical depth
#' @keywords internal
"df_netflux_0p4_lookup_v2"

#' Net flux polynomial coefficients (k=0)
#'
#' Polynomial coefficients for normalized net flux function calculation (albedo-independent terms).
#' Based on Appelbaum & Flood analytical expression.
#'
#' @format A data frame with polynomial coefficients
#' @keywords internal
"df_netflux_k0_coeffs"

#' Net flux polynomial coefficients (k=1)
#'
#' Polynomial coefficients for normalized net flux function calculation (albedo-dependent terms).
#' Based on Appelbaum & Flood analytical expression.
#'
#' @format A data frame with polynomial coefficients
#' @keywords internal
"df_netflux_k1_coeffs"

# (30) in (1993).
# FIXME:
#   1. For surface oriented towards the equator, gamma_c = 0 or 180/-180
#   2. For vertical surface (when Beta=90)
#   3. Not exactly what is in (21) to determine omega_rad.

#' Sunrise for inclined surface oriented toward equator
#'
#' Calculates sunrise hour angle for an inclined surface facing the equator.
#' Implements Equation 30 from Appelbaum (1993).
#'
#' @param phi Planetary latitude [rad]
#' @param beta Surface tilt angle [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Sunrise hour angle [rad]
#' @noRd
sunrise_for_inclined_surface_oriented_equator = function(phi, beta, delta){
  
  # (8) in (1993).
  omega_rad_1 = sunrise_for_horizontal_surface(phi=phi, delta=delta)

  # (30) in (1993).
  omega_rad_2 = -acos(-tan(delta) * tan(phi-beta))

  # From (21) in (1993) we want to grab the minium between (8) and (30).
  omega_rad = -min(abs(omega_rad_1), abs(omega_rad_2))
  
  # FIXME: Is it just (30)? This was checked with plots.
  #omega_rad = -acos(-tan(delta) * tan(phi-beta))
  
  return(omega_rad)
}

# (15) from (1993).
# Angles in rad.

#' Sunrise for inclined surface oriented eastward
#'
#' Calculates sunrise hour angle for an inclined surface facing east.
#' Implements Equation 15 from Appelbaum (1993).
#'
#' @param phi Planetary latitude [rad]
#' @param beta Surface tilt angle [rad]
#' @param gamma_c Surface azimuth angle [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Sunrise hour angle [rad], or NA if sun never rises on this surface
#' @noRd
sunrise_for_inclined_surface_oriented_east = function(phi, beta, gamma_c, delta){
  
  # Calculate omega for horizontal surface.
  omega_rad_1 = sunrise_for_horizontal_surface(phi=phi, delta=delta)

  # Calculate omega for inclined surface.
  x = x_for_inclined_surface(phi=phi, beta=beta, gamma_c=gamma_c)
  y = y_for_inclined_surface(phi=phi, beta=beta, gamma_c=gamma_c, delta=delta)

  # If the radicand is negative then it means that the sun never rises on the inclined surface.
  # This can be the case of the inclined surface is at a:
  #   - high planetary latitude and oriented northwards.
  #   - low planetary latitude and oriented southwards.
  radicand = x^2 - y^2 + 1
  if(radicand < 0){
    return(NA)
  }
  
  a = -x*y - sqrt(radicand)
  b = x^2 + 1
  
  omega_rad_2 = acos(a / b)
  
  # Pick between the two calculated omegas.
  omega_rad = -min(abs(omega_rad_1), omega_rad_2)
  
  return(omega_rad)
}


# (17) from (1993).
# Angles in rad.

#' Sunrise for inclined surface oriented westward
#'
#' Calculates sunrise hour angle for an inclined surface facing west.
#' Implements Equation 17 from Appelbaum (1993).
#'
#' @param phi Planetary latitude [rad]
#' @param beta Surface tilt angle [rad]
#' @param gamma_c Surface azimuth angle [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Sunrise hour angle [rad], or NA if sun never rises on this surface
#' @noRd
sunrise_for_inclined_surface_oriented_west = function(phi, beta, gamma_c, delta){
  
  # Calculate omega for horizontal surface.
  omega_rad_1 = sunrise_for_horizontal_surface(phi=phi, delta=delta)
  
  # Calculate omega for inclined surface.
  x = x_for_inclined_surface(phi=phi, beta=beta, gamma_c=gamma_c)
  y = y_for_inclined_surface(phi=phi, beta=beta, gamma_c=gamma_c, delta=delta)
  
  # If the radicand is negative then it means that the sun never rises on the inclined surface.
  # This can be the case of the inclined surface is at a:
  #   - high planetary latitude and oriented northwards.
  #   - low planetary latitude and oriented southwards.
  radicand = x^2 - y^2 + 1
  if(radicand < 0){
    return(NA)
  }
  
  a = -x*y + sqrt(radicand)
  b = x^2 + 1
  
  omega_rad_2 = acos(a / b)
  
  # Pick between the two calculated omegas.
  omega_rad = -min(abs(omega_rad_1), omega_rad_2)
  
  return(omega_rad)
}

# Angles in rad.

#' Sunrise for horizontal surface
#'
#' Calculates sunrise hour angle for a horizontal surface.
#' Implements Equation 8 from Appelbaum (1993).
#'
#' @param phi Planetary latitude [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Sunrise hour angle [rad]
#' @noRd
sunrise_for_horizontal_surface = function(phi, delta){
  
  # Equation 8 (1993): Sunrise hour angle [rad].
  omega_rad = -acos(-tan(delta) * tan(phi))
  
  return(omega_rad)
}

# Angles in rad.

#' Sunrise for inclined surface (general)
#'
#' Calculates sunrise hour angle for an inclined surface with any orientation.
#' Dispatches to specific orientation functions based on gamma_c value.
#'
#' @param phi Planetary latitude [rad]
#' @param beta Surface tilt angle [rad]
#' @param gamma_c Surface azimuth angle [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Sunrise hour angle [rad]
#' @noRd
sunrise_for_inclined_surface = function(phi, beta, gamma_c, delta){
  
  # Inclination angle is 0 degrees, this is equivalent to a horizontal surface.
  if(beta == 0){
    omega_rad = sunrise_for_horizontal_surface(phi=phi, delta=delta)
    
  }else if(gamma_c == 0){
    # Inclined surface facing the equator.
    # i.e. Oriented South when in the Northern hemisphere and oriented North when in the Southern Hemisphere.
    omega_rad = sunrise_for_inclined_surface_oriented_equator(phi=phi, beta=beta, delta=delta)
    
  }else if(round(abs(gamma_c), 2) == round(pi, 2)){
    # Inclined surface facing opposite the equator.
    # i.e. Oriented North when in the Northern hemisphere and Oriented South whn in the Southern Hemisphere.
    omega_rad = sunrise_for_inclined_surface_oriented_equator(phi=phi, beta=beta, delta=delta)
    
  }else if(gamma_c < 0){ # Inclined surface is oriented towards the East.
    omega_rad = sunrise_for_inclined_surface_oriented_east(phi=phi, beta=beta, gamma_c=gamma_c, delta=delta)
    
  }else if(gamma_c > 0){ # Incline surface is oriented towards the West.
    omega_rad = sunrise_for_inclined_surface_oriented_west(phi=phi, beta=beta, gamma_c=gamma_c, delta=delta)
    
  }else{
    # This should not happen.
    stop(paste("An unknown error has occurred. Could not determine sunrise hour angle for an inclined surface when phi=", phi, " beta=", beta, " and gamma_c=", gamma_c, ".", sep=""))
  } 
  
  return(omega_rad)
}

# The function.
#   Ls    - Areocentric longitude.
#   phi   - Planetary latitude.
#   unit  - Unit to return:
#           - 1 for radians.
#           - 2 for degrees.
#           - 3 for solar hour.
#' Sunrise time on Mars
#'
#' Calculates the sunrise time for a horizontal or inclined surface on Mars.
#' Returns NA during polar night/day periods.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param beta Surface tilt angle [deg]. Optional, for inclined surfaces
#' @param gamma_c Surface azimuth angle [deg]. Optional, for inclined surfaces. Zero facing equator, east negative, west positive (-180 to +180)
#' @param unit Output unit: 1 for radians, 2 for degrees, 3 for solar hours (default: 1)
#'
#' @return Sunrise time [rad], [deg], or [h] depending on unit parameter, or NA during polar night/day
#' @export
sunrise = function(Ls, phi, beta=NULL, gamma_c=NULL, unit=1){
  
  if(!unit %in% c(1, 2, 3)){
    stop("Sunrise unit option must either be 1 for radians, 2 for degrees, or 3 for solar hour.")
  }
  
  # The sunrise hour angle [rad] that will be calculated.
  omega_rad = NULL
  
  # Equation 7 (1990): Declination angle [rad].
  delta_rad = declination(Ls)
  
  # Equations 16 (Update 1991): Figure out if it is polar night or polar day.
  # For polar nights and polar days:
  #   Polar night (polar_flag < -1), no solar irradiance.
  #   Polar day (polar_flag > 1), constant solar irradiance. 
  polar_flag = -tan(delta_rad) * tan(phi*pi/180)
  
  # If polar night or polar day, then there is no sunrise or sunset.
  if(polar_flag < -1 || polar_flag > 1){
    warning("Trying to get a sunrise hour angle during a polar night. This returns an NA which must be handled properly.")
    return(NA);
  }
  
  if((is.null(beta) && is.null(gamma_c))){
    omega_rad = sunrise_for_horizontal_surface(phi=phi*pi/180, delta=delta_rad)
    
  }else if(!is.null(beta) && !is.null(gamma_c)){
    if(gamma_c > 180 || gamma_c < -180){
      stop("Surface azimuth angle gamma_c must between -180 and 180 degrees with zero south, east negative, and west positive.")
    }
    
    omega_rad = sunrise_for_inclined_surface(phi=phi*pi/180, beta=beta*pi/180, gamma_c=gamma_c*pi/180, delta=delta_rad)

  }else{
    stop("Invalid argument values. Both beta and gamma_c should either be NULL or not NULL.")
  }
  
  # If the radicand is negative then it means that the sun never rises on the inclined surface.
  # This can be the case of the inclined surface is at a:
  #   - high planetary latitude and oriented northwards.
  #   - low planetary latitude and oriented southwards.
  if(is.na(omega_rad)){
    return(NA)
  }
  
  # Get the hour angle in degrees.
  omega_deg = omega_rad * 180/pi
  
  # Equation 8 (1990): Hour angle. Determine the Surise and Sunset times [h].
  # From Appelbaum, Joseph & Flood, Dennis. (1990):
  #   The ratio of Mars to Earth length of day is 24.65/24.
  #   It is convenient, for calculation purposes, to define a Mar hour
  #   by dividing the Martian day into 24 hr. Using the same relationship
  #   between the Mars solar time T and the hour angle as for the Earth.
  Ts = (omega_deg + 180) / 15
  
  if(unit == 1){
    return(omega_rad)
    
  }else if(unit == 2){
    return(omega_deg)
    
  }else if(unit == 3){
    return(Ts)
    
  }else{
    # This should not happen.
    stop("An unknown error has occurred.")
  }  
}

# Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)
# 
# Ls_seq=1:360
# Ls=27
# phi=34
# 
# #beta=45
# gamma_c=140
# 
# Ls = 1
# Ls_seq = 1:360
# phi = -10
# beta = 1
# gamma_c = 0
# 
# index = 1
# for(gamma_c in gamma_c){
# #for(phi in seq(45, 50, 5)){
# 
#   sr_vect = c()
#   ss_vect = c()
#   for(Ls in Ls_seq){
# 
#     delta_rad = declination(Ls)
#     beta = optimal_angle(Ls=Ls, phi=phi, unit=2)
# 
#     #cs = constrain_solar_time_range(Ls=Ls, phi=phi, Ts_start=0, Ts_end=24, beta=beta, gamma_c=gamma_c)
# 
#     #print(paste("Ls ", Ls, ", phi ", phi, ", beta ", beta))
#     #print(cs)
# 
#     sr = sunrise(Ls=Ls, phi=phi, beta=beta, gamma_c=gamma_c, unit=3)
#     ss = sunset(Ls=Ls, phi=phi, beta=beta, gamma_c=gamma_c, unit=3)
#     sr_vect = c(sr_vect, sr)
#     ss_vect = c(ss_vect, ss)
#     #print(paste(gamma_c, " ", sr))
# 
#   }
# 
#   if(index == 1){
#     dev.new()
#     plot(Ls_seq, sr_vect, ylim=c(0,20),
#          type="l", lwd=2, xlab="Areocentric Longitude [deg]", ylab="Surface sunrise time [hr]")
# 
#     lines(Ls_seq, ss_vect, lwd=2)
#   }else{
#     lines(Ls_seq, sr_vect, lwd=2)
#     lines(Ls_seq, ss_vect, lwd=2)
#   }
# 
#   index = index + 1
# }

context("Sunrise")

# For a given planetary latitude, the sunrise time is expected to be the same throughout the entire year
# for all orientations when on a horizontal surface.
test_that("Sunrise time on an horizontal surface facing 'all' orientations.", {

  sunrise_hours <<- c()

  for(Ls in seq(0, 360, 20)){
    for(phi in seq(-90, 90, 10)){
      for(gamma_c in seq(-180, 180, 10)){
        sr = sunrise(Ls=Ls, phi=phi, beta=0, gamma_c=gamma_c, unit=3)
        sunrise_hours <<- c(sunrise_hours, sr)
      }

      expect_length(unique(sunrise_hours), 1)
      sunrise_hours <<- c()
    }
  }
})

#TODO: Test cases:
#   1. At the equator, when phi = 0. omega_sr = pi/2.
#   2. At the equinox, when delta = 0. omega_sr = pi/2.
test_that("Sunrise time on an inclined surface facing the equator.", {

  # Tolerance
  tolerance = 0.01

  # Sunrise data to test against. For beta = phi.
  expected_sunrise_times = read.csv("data/sunrise_beta_equals_phi_table_i_1993.csv")

  # Rename rows labels (Ls).
  rownames(expected_sunrise_times) = sprintf("%i", expected_sunrise_times[,1])
  expected_sunrise_times = expected_sunrise_times[-c(1)]

  # Rename column labels (beta/phi). Use 'm' for minus/negative and 'p' for plus/positive
  colnames(expected_sunrise_times) = gsub("X\\.", "m", colnames(expected_sunrise_times))
  colnames(expected_sunrise_times) = gsub("X", "p", colnames(expected_sunrise_times))
  colnames(expected_sunrise_times) = gsub("p0", "0", colnames(expected_sunrise_times))

  # Functo to fetch sunrise time from expected sunrise table.
  get_expected_sunrise_time = function(Ls, beta_phi){

    # Turn Ls value into row label value.
    Ls = toString(Ls)

    # Turn beta/phi angle numeric value into column label string value.
    if(beta_phi == 0){
      beta_phi = toString(beta_phi)

    }else if(beta_phi > 0){
      beta_phi = paste("p", beta_phi, sep="")

    }else if(beta_phi < 0){
      beta_phi = paste("m", abs(beta_phi), sep="")
    }

    # Get sunrise hour.
    sunrise = expected_sunrise_times[Ls, beta_phi]

    # For expected sunrise times, the 6.00 and 12.00 times for high altitudes
    # indicate polar days and polar nights respectively (see 1993).
    if(beta_phi >= 70){
      if(sunrise %in% c(6, 12)){
        return(NA)
      }
    }

    # Return sunrise hour.
    return(sunrise)
  }

  # Assert equals test against all values in the expected sunrise table.
  for(Ls in seq(0, 360, 5)){

    # The expected data was obtained with Phi = Beta. See Table I in (1993).
    for(phi_beta in seq(-90, 90, 10)){

      # Make sure that orientation is towards the equator because that is what we are testing.
      # If on a flat horizontal surface, i.e. beta = 0, then it doesn't matter what the orientation is (i.e. gamma_c).
      if(phi_beta > 0){
        # Located in the northern hemisphere: orient towards the south.
        gamma_c_vect = c(0)

      }else if(phi_beta < 0){
        # Located in the southern hemisphere: orient towards the north
        gamma_c_vect = c(-180, 180)
      }

      for(gamma_c in gamma_c_vect){
        # Get expected sunrise time from test table.
        sunrise_expected = get_expected_sunrise_time(Ls, phi_beta)

        # Calculate sunrise time.
        sunrise_calculated = sunrise(Ls=Ls, phi=phi_beta, beta=phi_beta, gamma_c=gamma_c, unit=3)

        # Due to floating point precision, days that are calculated to be just on the edge of polar day may not match with
        # expected results which state that they are exactly on polar days.
        # This workaround addresses that.
        # FIXME: Why is this not an issue with polar nights?
        if(is.na(sunrise_expected) && !is.na(sunrise_calculated) && round(sunrise_calculated, 4) == 6){
          sunrise_expected = 6
        }

        #print(paste("Ls:", Ls, "- phi/beta:", phi_beta, "- gamma_c:", gamma_c, "- sunrise_c:", sunrise_calculated, "- sunrise_e:", sunrise_expected))
        expect_equal(sunrise_calculated, sunrise_expected, tolerance=tolerance, scale=1)
      }
    }
  }
})

# FIXME: NaNs are produced when calculating sqrt(x^2 - y^2 + 1) in some cases for large beta angles (for beta >= 40 when +30 > phi > -30).
#        Where? In sunrise_for_inclined_surface_oriented_east and sunrise_for_inclined_surface_oriented_west:
#       x ^2 - y^2 + 1 results in a negative number thus outputting NaN when calling sqrt().        
# test_that("Sunrise on an inclined surface - Exhaustive Warnings Test.", {
#   Ls_issues = c()
#   phi_issues = c()
#   beta_issues = c()
#   gamma_issues = c()
# 
#   # for(Ls in seq(0, 360, 80)){
#   #   for(p in seq(-30, 30, 10)){
#   #     for(b in seq(0, 80, 5)){
#   #       for(gc in seq(-180, 180, 5)){
#           
#   for(Ls in c(80)){
#     for(p in seq(30, 30, 10)){
#       for(b in seq(80, 80, 5)){
#         for(gc in seq(-180, 180, 5)){
# 
#           tryCatch(
#             {
#               Tsr = NULL
#               Tsr_i = NULL
# 
#               Tsr = sunrise(Ls=Ls, phi=p, beta=NULL, gamma_c=NULL, unit=3)
#               Tsr_i = sunrise(Ls=Ls, phi=p, beta=b, gamma_c=gc, unit=3)
#             },
#             error=function(cond) {
#               # cat("\n============================================================")
#               # cat(paste("\n\nError:", cond))
#               # cat(paste("Tsr: ", Tsr, ",    Tsr_i:", Tsr_i))
#               # cat(paste("\nLs=", Ls, "phi=, ", p, ", beta=", b, ", gamma_c=", gc, "\n", sep=""))
#             },
#             warning=function(cond) {
#               # cat("\n============================================================")
#               # cat(paste("\n\nWarning:", cond))
#               # cat(paste("Tsr: ", Tsr, ",    Tsr_i:", Tsr_i))
#               # cat(paste("\nLs=", Ls, ", phi=", p, ", beta=", b, ", gamma_c=", gc, "\n", sep=""))
#               
#               # Enter here when NaNs produced from sqrt(x^2 - y^2 + 1).
# 
#               Ls_issues <<- c(Ls_issues, Ls)
#               phi_issues <<- c(phi_issues, p)
#               beta_issues <<- c(beta_issues, b)
#               gamma_issues <<- c(gamma_issues, gc)
#             }
#           )
#         }
#       }
#     }
#   }
# 
#   # Expect no warnings triggered.
#   expect_length(Ls_issues, 0)
# 
#   if(length(Ls_issues) != 0){
#     cat("\nProblematic variables:\n")
#     cat(paste("\n\tLs:", paste(unique(Ls_issues), collapse=", ")))
#     cat(paste("\n\tphi:", paste(unique(phi_issues), collapse=", ")))
#     cat(paste("\n\tbeta:", paste(unique(beta_issues), collapse=", ")))
#     cat(paste("\n\tgamma_c:", paste(unique(gamma_issues), collapse=", ")))
#   }
# })

# FIXME: Bad logic when gamma_c = 0 and Beta > 0.

#' Sunset for horizontal surface
#'
#' Calculates sunset hour angle for a horizontal surface.
#' Implements Equation 9 from Appelbaum (1993).
#'
#' @param phi Planetary latitude [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Sunset hour angle [rad]
#' @noRd
sunset_for_horizontal_surface = function(phi, delta){

  # (9) in (1993): Sunset hour angle [rad].
  omega_rad = acos(-tan(delta) * tan(phi))
  
  return(omega_rad)
}


# FIXME: Doesn't work for all cases, see Ls = 300, phi = 20, and beta = 45.

#' Sunset for inclined surface oriented toward equator
#'
#' Calculates sunset hour angle for an inclined surface facing the equator.
#' Implements Equation 31 from Appelbaum (1993).
#'
#' @param phi Planetary latitude [rad]
#' @param beta Surface tilt angle [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Sunset hour angle [rad]
#' @noRd
sunset_for_inclined_surface_oriented_equator = function(phi, beta, delta){

  # (9) in (1993).
  omega_rad_1 = sunset_for_horizontal_surface(phi=phi, delta=delta)

  # (31) in (1993).
  omega_rad_2 = acos(-tan(delta) * tan(phi-beta))

  # From (22) in (1993) we want to grab the minium between (8) and (30).
  omega_rad = min(omega_rad_1, omega_rad_2)
  
  # FIXME: Is it just (31)? This was checked with plots.
  #omega_rad = acos(-tan(delta) * tan(phi-beta))
  
  return(omega_rad)
}

# (16) from (1993).

#' Sunset for inclined surface oriented eastward
#'
#' Calculates sunset hour angle for an inclined surface facing east.
#' Implements Equation 16 from Appelbaum (1993).
#'
#' @param phi Planetary latitude [rad]
#' @param beta Surface tilt angle [rad]
#' @param gamma_c Surface azimuth angle [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Sunset hour angle [rad], or NA if sun never sets on this surface
#' @noRd
sunset_for_inclined_surface_oriented_east = function(phi, beta, gamma_c, delta){

  omega_rad_1 = sunset_for_horizontal_surface(phi=phi, delta=delta)
  
  x = x_for_inclined_surface(phi=phi, beta=beta, gamma_c=gamma_c)
  y = y_for_inclined_surface(phi=phi, beta=beta, gamma_c=gamma_c, delta=delta)
  
  # If the radicand is negative then it means that the sun never sets on the inclined surface.
  # This can be the case of the inclined surface is at a:
  #   - high planetary latitude and oriented northwards.
  #   - low planetary latitude and oriented southwards.
  radicand = x^2 - y^2 + 1
  if(radicand < 0){
    return(NA)
  }
  
  a = -x*y + sqrt(radicand)
  b = x^2 + 1
  
  omega_rad_2 = acos(a / b)
  
  omega_rad = min(omega_rad_1, omega_rad_2)
  
  return(omega_rad)
}

# (18) from (1993).

#' Sunset for inclined surface oriented westward
#'
#' Calculates sunset hour angle for an inclined surface facing west.
#' Implements Equation 18 from Appelbaum (1993).
#'
#' @param phi Planetary latitude [rad]
#' @param beta Surface tilt angle [rad]
#' @param gamma_c Surface azimuth angle [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Sunset hour angle [rad], or NA if sun never sets on this surface
#' @noRd
sunset_for_inclined_surface_oriented_west = function(phi, beta, gamma_c, delta){

  omega_rad_1 = sunset_for_horizontal_surface(phi=phi, delta=delta)
  
  x = x_for_inclined_surface(phi=phi, beta=beta, gamma_c=gamma_c)
  y = y_for_inclined_surface(phi=phi, beta=beta, gamma_c=gamma_c, delta=delta)
  
  # If the radicand is negative then it means that the sun never sets on the inclined surface.
  # This can be the case of the inclined surface is at a:
  #   - high planetary latitude and oriented northwards.
  #   - low planetary latitude and oriented southwards.
  radicand = x^2 - y^2 + 1
  if(radicand < 0){
    return(NA)
  }
  
  a = -x*y - sqrt(radicand)
  b = x^2 + 1
  
  omega_rad_2 = acos(a / b)
  
  omega_rad = min(omega_rad_1, omega_rad_2)
  
  return(omega_rad)
}

#' Sunset for inclined surface (general)
#'
#' Calculates sunset hour angle for an inclined surface with any orientation.
#' Dispatches to specific orientation functions based on gamma_c value.
#'
#' @param phi Planetary latitude [rad]
#' @param beta Surface tilt angle [rad]
#' @param gamma_c Surface azimuth angle [rad]
#' @param delta Solar declination angle [rad]
#'
#' @return Sunset hour angle [rad]
#' @noRd
sunset_for_inclined_surface = function(phi, beta, gamma_c, delta){

  # Inclination angle is 0 degrees, this is equivalent to a horizontal surface.
  if(beta == 0){
    omega_rad = sunset_for_horizontal_surface(phi=phi, delta=delta)
    
  }else if(gamma_c == 0){
    # Inclined surface facing the equator.
    # i.e. Oriented South when in the Northern hemisphere and oriented North when in the Southern Hemisphere.
    omega_rad = sunset_for_inclined_surface_oriented_equator(phi=phi, beta=beta, delta=delta)
    
  }else if(round(abs(gamma_c), 2) == round(pi, 2)){
    # Inclined surface facing opposite the equator.
    # i.e. Oriented North when in the Northern hemisphere and Oriented South whn in the Southern Hemisphere.
    omega_rad = sunset_for_inclined_surface_oriented_equator(phi=phi, beta=beta, delta=delta)
    
  }else if(gamma_c < 0){ # Inclined surface is oriented towards the East.
    omega_rad = sunset_for_inclined_surface_oriented_east(phi=phi, beta=beta, gamma_c=gamma_c, delta=delta)
    
  }else if(gamma_c > 0){ # Incline surface is oriented towards the West.
    omega_rad = sunset_for_inclined_surface_oriented_west(phi=phi, beta=beta, gamma_c=gamma_c, delta=delta)
    
  }else{
    # This should not happen.
    stop(paste("An unknown error has occurred. Could not determine sunset hour angle for an inclined surface when phi=", phi, " beta=", beta, " and gamma_c=", gamma_c, ".",  sep=""))
  } 
  
  return(omega_rad)
}


#' Sunset time on Mars
#'
#' Calculates the sunset time for a horizontal or inclined surface on Mars.
#' Returns NA during polar night/day periods.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param beta Surface tilt angle [deg]. Optional, for inclined surfaces
#' @param gamma_c Surface azimuth angle [deg]. Optional, for inclined surfaces. Zero facing equator, east negative, west positive (-180 to +180)
#' @param unit Output unit: 1 for radians, 2 for degrees, 3 for solar hours (default: 1)
#'
#' @return Sunset time [rad], [deg], or [h] depending on unit parameter, or NA during polar night/day
#' @export
sunset = function(Ls, phi, beta=NULL, gamma_c=NULL, unit=1){
  
  if(!unit %in% c(1, 2, 3)){
    stop("Sunrise unit option must either be 1 for radians, 2 for degrees, or 3 for solar hour.")
  }
  
  # The sunrise hour angle [rad] that will be calculated.
  omega_rad = NULL
  
  # Equation 7 (1990): Declination angle [rad].
  delta_rad = declination(Ls)
  
  # Equations 16 (Update 1991): Figure out if it is polar night or polar day.
  # For polar nights and polar days:
  #   Polar night (polar_flag < -1), no solar irradiance.
  #   Polar day (polar_flag > 1), constant solar irradiance. 
  polar_flag = -tan(delta_rad) * tan(phi*pi/180)
  
  # If polar night or polar day, then there is no sunrise or sunset.
  if(polar_flag < -1 || polar_flag > 1){
    warning("Trying to get a sunset hour angle during a polar night. This returns an NA which must be handled properly.")
    return(NA);
  }
  
  if((is.null(beta) && is.null(gamma_c))){
    omega_rad = sunset_for_horizontal_surface(phi=phi*pi/180, delta=delta_rad)
    
  }else if(!is.null(beta) && !is.null(gamma_c)){
    if(gamma_c > 180 || gamma_c < -180){
      stop("Surface azimuth angle gamma_c must between -180 and 180 degrees with zero south, east negative, and west positive.")
    }
    
    omega_rad = sunset_for_inclined_surface(phi=phi*pi/180, beta=beta*pi/180, gamma_c=gamma_c*pi/180, delta=delta_rad)
    
  }else{
    stop("Invalid argument values. Both beta and gamma_c should either be NULL or not NULL.")
  }
  
  if(is.null(omega_rad)){
    stop("An unknown error has occurred. The sunset hour angle has not been set.")
  }
  
  # Get the hour angle in degrees.
  omega_deg = omega_rad * 180/pi
  
  # Equation 8 (1990): Hour angle. Determine the Surise and Sunset times [h].
  # From Appelbaum, Joseph & Flood, Dennis. (1990):
  #   The ratio of Mars to Earth length of day is 24.65/24.
  #   It is convenient, for calculation purposes, to define a Mar hour
  #   by dividing the Martian day into 24 hr. Using the same relationship
  #   between the Mars solar time T and the hour angle as for the Earth.
  Ts = (omega_deg + 180) / 15
  
  if(unit == 1){
    return(omega_rad)
    
  }else if(unit == 2){
    return(omega_deg)
    
  }else if(unit == 3){
    return(Ts)
    
  }else{
    # This should not happen.
    stop("An unknown error has occurred.")
  }  
}

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

test_that("G_b: Direct beam irradiance normal to solar rays", {
  # Test parameters
  Ls = 90     # Areocentric longitude (northern summer solstice)
  phi = 22.48 # Viking 1 latitude
  Ts = 12     # Solar time (noon)
  tau = 0.5   # Optical depth

  # Test normal irradiated case
  result = G_b(Ls = Ls, phi = phi, Ts = Ts, tau = tau)
  expect_true(is.numeric(result))
  expect_true(result > 0)

  # Test with explicit zenith angle
  z = Z(Ls = Ls, phi = phi, Ts = Ts)
  result2 = G_b(Ls = Ls, z = z, tau = tau)
  expect_equal(result, result2)

  # Test nighttime (not irradiated) - should return 0
  # Use a time well after sunset (Ts = 0 is midnight)
  result_night = G_b(Ls = Ls, phi = phi, Ts = 0, tau = tau)
  expect_equal(result_night, 0)

  # Test with sun below horizon (z >= 90)
  result_below = G_b(Ls = Ls, z = 95, tau = tau)
  expect_equal(result_below, 0)
})

# FIXME: Update this function so that it figures out if its a polar night or day.

# Equation 16 (1990):  The diffuse irradiance on a horizontal Martian surface.
#
# Gh = Gbh + Gdh
# Determine an expression for the diffuse irradiance based on Eq. 17 and Eq. 18.
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252


#' Diffuse irradiance on Mars horizontal surface
#'
#' Calculates the diffuse solar irradiance (scattered by atmospheric dust) incident on a
#' horizontal surface on Mars. Computed as the difference between global and direct beam
#' irradiance. Implements Equation 16 from Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param Ts Solar time [h]. Can be NULL for daily calculations
#' @param z Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts
#' @param tau Atmospheric optical depth (dimensionless)
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#'
#' @return Diffuse irradiance on horizontal surface [W/m²]
#' @export
G_dh = function(Ls, phi, longitude, Ts=NULL, z=Z(Ls=Ls, phi=phi, Ts=Ts), tau, al=albedo(latitude=phi, longitude=longitude, tau=tau)){
  # TODO: What happens when Ts is NULL? How is z calculated?
  if(!is_irradiated(Ls=Ls, phi=phi, Ts=Ts, z=z)){
    return(0)
    
  }else{
    Gdh = G_h(Ls=Ls, phi=phi, longitude=longitude, Ts=Ts, z=z, tau=tau, al=al) - G_bh(Ls=Ls, phi=phi, Ts=Ts, z=z, tau=tau)
    return(Gdh)
  }
} 

# Equation 11 (1990): Beam insolation on a horizotal surface at the top of Mars atmosphere [Wh/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252


#' Beam insolation at top of Mars atmosphere over time period
#'
#' Calculates the solar beam energy on a horizontal surface at the top of the Martian atmosphere
#' (before atmospheric attenuation) between specified start and end times. Implements Equation 11
#' from Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param Ts_start Start of integration period [h]
#' @param Ts_end End of integration period [h]
#'
#' @return Beam insolation at top of atmosphere over specified time period [Wh/m²]
#' @export
I_obh = function(Ls, phi, Ts_start, Ts_end){
  
  # Step 1: Constrain Ts_start and Ts_end based on sunrise and sunset times.
  
  # Apply solar time range constraint.
  T_range = constrain_solar_time_range(Ls=Ls, phi=phi, Ts_start=Ts_start, Ts_end=Ts_end)

  # No solar irradiance within the contrained time range.
  if(is.null(T_range)){
    return(0)

  }else{
    # Constrain the time range.
    Ts_start = T_range$Ts_start
    Ts_end = T_range$Ts_end
  }
  
  # Step 2: Calculate insolation.
  
  # The integrand for Equation 11 (1990).
  integrand = function(Ts){
    z = Z(Ls=Ls, phi=phi, Ts=Ts)

    x = G_ob(Ls) * cos(z*pi/180)
    return(x)
  }

  # Equation 11 (1990): Beam insolation on a horizontal surface at the top of Mars atmosphere [Wh/m2].
  Iobh = integrate(integrand, Ts_start, Ts_end)

  return(Iobh$value)
}

# Global daily insolation on Mars horizontal surface [Wh/m2-day].
#
# Obtained by integrating I_h, global hourly insolation on Mars horizontal surface, 
# over the period from sunrise to sunset.
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252


#' Global daily insolation on Mars horizontal surface
#'
#' Calculates the total solar energy received over a full Martian day on a horizontal surface.
#' Obtained by integrating global hourly insolation from sunrise to sunset.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param tau Atmospheric optical depth (dimensionless)
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#'
#' @return Global daily insolation [Wh/m²-day]
#' @export
H_h = function(Ls, phi, longitude, tau, al=albedo(latitude=phi, longitude=longitude, tau=tau)){

  # Hh is obtained by integrating Ih over the period from sunrise to sunset.
  Hh = I_h(Ls=Ls, phi=phi, longitude=longitude, tau=tau, Ts_start=0, Ts_end=24, al=al)
  
  # Return result.
  return(Hh)
}
              
              
              

# Global daily insolation on Mars horizontal surface [Wh/m2-day].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252
#
# Expected results taken from:
#   TABLE IV. - HOURLY AND DAILY GLOBAL INSOLATION ON A HORIZONTAL SURFACE AT MARS SURFACE.
#   TABLE V. Daily insolation on a horizontal surface (Update 1990).
context("Global daily insolation on Mars horizontal surface")
source("utils.R")

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

# Test with expected results from TABLE IV.
test_that("H_h: Global daily insolation on a horizontal surface", {
  tolerance = 25

  phi = 22.3
  al = 0.1

  expected_results = list(
    "69" = c(0.65, 3340),
    "120" = c(0.40, 3860),
    "153" = c(0.50, 3882),
    "249" = c(1.40, 1900))
    # FIXME: Larger errors when Ls = 299°.
    #"299" = c(3.25, 1024))

  index = 1
  for(er in expected_results){

    Ls = strtoi(names(expected_results)[index])
    tau = er[1]
    Hh_expected = er[2]

    Hh = H_h(Ls=Ls, phi=phi, tau=tau, al=al)
    expect_equal(Hh, Hh_expected, tolerance=tolerance, scale=1)

    index = index + 1
  }
})

test_that("H_h: Global daily insolation on a horizontal surface at VL1", {
  # 3.2% Error tolerance.
  tol = 0.032

  # Test input parameter.
  Ls_seq = seq(0, 355, 5)
  al = 0.1

  # Run the tests.
  test_daily_insolation_on_horizontal_surface(
    spacecraft = "VL1",
    field = "Hh",
    tolerance = tol,
    Ls_seq = Ls_seq,
    al = al,
    verbose = FALSE)

})

test_that("H_h: Global daily insolation on a horizontal surface at VL2", {
  # 3.8% Error tolerance.
  tol = 0.038

  # Test input parameter.
  Ls_seq = seq(0, 355, 5)
  al = 0.1

  # Run the tests.
  test_daily_insolation_on_horizontal_surface(
    spacecraft = "VL2",
    field = "Hh",
    tolerance = tol,
    Ls_seq = Ls_seq,
    al = al,
    verbose = FALSE)

})

#' Diffuse insolation on Mars inclined surface over time period
#'
#' Calculates the diffuse solar energy (scattered by atmospheric dust) received on an inclined
#' surface between specified start and end times. Obtained by integrating diffuse irradiance over the time period.
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param longitude Planetary longitude [deg]
#' @param tau Atmospheric optical depth (dimensionless)
#' @param Ts_start Start of integration period [h]
#' @param Ts_end End of integration period [h]
#' @param al Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau
#' @param beta Surface tilt/slope angle from horizontal [deg]
#' @param gamma_c Surface azimuth angle [deg]. Zero facing equator, east negative, west positive (-180 to +180)
#'
#' @return Diffuse insolation on inclined surface over specified time period [Wh/m²]
#' @export
I_di = function(Ls, phi, longitude, tau, Ts_start, Ts_end, al=albedo(latitude=phi, longitude=longitude, tau=tau), beta, gamma_c){
  
  if(gamma_c > 180 || gamma_c < -180){
    stop("Surface azimuth angle gamma_c must between -180\u00b0 and +180\u00b0 with zero south, east negative, and west positive.")
  }
  
  # Step 1: Constrain Ts_start and Ts_end based on sunrise and sunset times.
  
  # Apply solar time range constraint.
  T_range = constrain_solar_time_range(Ls, phi, Ts_start, Ts_end, beta, gamma_c)
  
  # No solar irradiance within the contrained time range.
  if(is.null(T_range)){
    return(0)
    
  }else{
    # Constrain the time range.
    Ts_start = T_range$Ts_start
    Ts_end = T_range$Ts_end
  }
  
  # Step 2: Calculate insolation.

  integrand = function(Ts){
    Gdi = G_di(Ls=Ls, phi=phi, longitude=longitude, Ts=Ts, tau=tau, al=al, beta=beta)
    return(Gdi)
  }
  
  Idi = integrate(integrand, Ts_start, Ts_end)
  
  # Return integration result.
  return(Idi$value)
}

# FIXME: Update this function so that it figures out if its a polar night or day.

# Equation 18: Beam irradiance on Mars horizontal surface [W/m2].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252


#' Direct beam irradiance on Mars horizontal surface
#'
#' Calculates the direct beam solar irradiance incident on a horizontal surface on Mars.
#' Accounts for the angle of incidence on the horizontal plane. Implements Equation 18
#' from Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param Ts Solar time [h]
#' @param z Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts
#' @param tau Atmospheric optical depth (dimensionless)
#'
#' @return Direct beam irradiance on horizontal surface [W/m²]
#' @export
G_bh = function(Ls, phi, Ts, z=Z(Ls=Ls, phi=phi, Ts=Ts), tau){
  
  if(!is_irradiated(Ls=Ls, phi=phi, Ts=Ts, z=z)){
    return(0)
    
  }else{
    G_ob(Ls) * cos(z * pi/180) * exp(-tau / cos(z*pi/180))
  }
} 

#' Solar zenith angle
#'
#' Calculates the angle between the sun's rays and the vertical (zenith) direction.
#' A zenith angle of 0° means the sun is directly overhead, while 90° means the sun
#' is at the horizon. Implements Equation 6 from Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#' @param Ts Solar time [h]
#'
#' @return Sun zenith angle [deg]
#' @export
Z = function(Ls, phi, Ts){
  
  # Equation 7: Declination angle [rad].
  delta = declination(Ls)
  
  # Equation 8: Hour angle [deg].
  omega = 15 * Ts - 180
  
  # Equation 6: Zenith angle of the incident solar radiation [deg].
  a = sin(phi*pi/180) * sin(delta)
  b = cos(phi*pi/180) * cos(delta) * cos(omega * pi/180)
  
  # Calculate z.
  z = acos(a + b) * 180/pi
  
  # Return result.
  return(z)
}

#' marsrad: Mars Solar Radiation
#'
#' @description
#' A set of functions to calculate solar irradiance and insolation on Mars horizontal
#' and inclined surfaces. Based on NASA Technical Memoranda 102299, 103623, 105216,
#' 106321, and 106700, i.e. the canonical Mars solar radiation papers.
#'
#' @details
#' The package provides three types of solar radiation calculations:
#' \itemize{
#'   \item Instantaneous irradiance (G_* functions) — Power per unit area at a specific moment
#'   \item Daily insolation (H_* functions) — Energy per unit area over one Martian sol
#'   \item Period insolation (I_* functions) — Energy per unit area over multiple sols
#' }
#'
#' All calculations support both horizontal and inclined surfaces. The package includes
#' functions for optimal tilt angle calculation, sunrise/sunset times, and atmospheric
#' optical depth modeling.
#'
#' @section Package Information:
#' \describe{
#'   \item{Version}{1.0.0}
#'   \item{License}{GPL-3}
#'   \item{Website}{\url{https://georges.fyi/marsrad/}}
#'   \item{Author}{\href{https://georges.fyi}{Georges Labrèche}}
#'   \item{Affiliation}{\href{https://tanagraspace.com}{Tanagra Space}}
#'   \item{Depends}{R (>= 2.10)}
#' }
#'
#' @section Citation:
#' If you use this package in your research or publication, please cite the paper it was developed for:
#'
#' Labrèche, G., & Cordes, F. (2020). Using a Rover's Active Suspension System as a 2-Axis Solar Tracker Mechanism.
#' \emph{15th International Symposium on Artificial Intelligence, Robotics and Automation in Space (i-SAIRAS '20)}.
#' \url{https://www.hou.usra.edu/meetings/isairas2020fullpapers/pdf/5035.pdf}
#'
#' @section Links:
#' \describe{
#'   \item{Package website}{\url{https://georges.fyi/marsrad/}}
#'   \item{Package GitHub repository}{\url{https://github.com/georgeslabreche/marsrad}}
#'   \item{Author LinkedIn}{\url{https://www.linkedin.com/in/georgeslabreche/}}
#'   \item{Author website}{\url{https://georges.fyi}}
#' }
#'
#' @examples
#' # Calculate horizontal irradiance at Viking 1 landing site on Ls 90 (northern summer solstice)
#' G_h(Ls = 90, phi = 22.48, longitude = -48, Ts = 12, tau = 0.5)
#'
#' # Find optimal panel tilt angle for the same location and season
#' optimal_angle(Ls = 90, phi = 22.48)
#'
#' # Calculate daily insolation on a horizontal surface
#' H_h(Ls = 90, phi = 22.48, longitude = -48, tau = 0.5)
#'
#' # Calculate insolation over a 24-hour period on an inclined surface
#' I_i(Ls = 90, phi = 22.48, longitude = -48, tau = 0.5, Ts_start = 0,
#'     Ts_end = 24, beta = 25, gamma_c = 0)
#'
#' @references
#' Appelbaum, J., & Flood, D. J. (1989). Solar Radiation on Mars.
#' NASA Technical Memorandum 102299.
#' \url{https://ntrs.nasa.gov/citations/19890018252}
#'
#' Appelbaum, J., & Flood, D. J. (1990). Solar radiation on Mars: Update 1990.
#' NASA Technical Memorandum 103623.
#' \url{https://ntrs.nasa.gov/citations/19910005804}
#'
#' Appelbaum, J., & Flood, D. J. (1991). Solar radiation on Mars: Update 1991.
#' NASA Technical Memorandum 105216.
#' \url{https://ntrs.nasa.gov/citations/19910023732}
#'
#' Appelbaum, J., Sherman, I., & Landis, G. A. (1993).
#' Solar radiation on Mars: Stationary photovoltaic array.
#' NASA Technical Memorandum 106321.
#' \url{https://ntrs.nasa.gov/citations/19940010257}
#'
#' Appelbaum, J., Flood, D. J., & Norambuena, M. (1994).
#' Solar radiation on Mars: Tracking photovoltaic array.
#' NASA Technical Memorandum 106700.
#' \url{https://ntrs.nasa.gov/citations/19950004977}
#'
#' @seealso
#' Labrèche, G. (2020). Exploiting the SherpaTT Rover Active Suspension System to Enable
#' Optimal Solar Array Inclination and Orientation for Long Traverses in a Martian Environment.
#' \emph{Master's Thesis, Luleå University of Technology}.
#' \url{https://www.diva-portal.org/smash/record.jsf?pid=diva2:1413245}
#' 
"_PACKAGE"
#' @importFrom stats integrate
NULL

# Equation 14 (1990): Direct beam irradiance on the Mars surface normal to the solar rays.
#
# From Appelbaum, Joseph & Flood, Dennis. (1990):
#   The direct beam irradiance, Gb, on the Martian surface normal to the solar rays
#   is related by Beer's law to the optical depth, tau, of the intervening atmospheric
#   haze [...] where m(z) is the air mass determined by the zenith angle Z.
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252


#' Direct beam irradiance on Mars surface normal to solar rays
#'
#' Calculates the direct beam solar irradiance on the Martian surface normal to the solar rays
#' (i.e., perpendicular to the sun's direction). Uses Beer's law to account for atmospheric
#' attenuation. Implements Equation 14 from Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]. Can be NULL if z is provided
#' @param Ts Solar time [h]. Can be NULL if z is provided
#' @param z Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts
#' @param tau Atmospheric optical depth (dimensionless)
#'
#' @return Direct beam irradiance normal to sun [W/m²]
#' @export
G_b = function(Ls, phi=NULL, Ts=NULL, z=Z(Ls=Ls, phi=phi, Ts=Ts), tau){
  
  if(!is_irradiated(Ls=Ls, phi=phi, Ts=Ts, z=z)){
    return(0)
    
  }else{
    G_ob(Ls) * exp(-tau / cos(z*pi/180))
  }
}

# Equation 10 (1990): Number of Mars daylight hours.
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://www.researchgate.net/publication/256334925_Solar_radiation_on_Mars


#' Number of Mars daylight hours
#'
#' Calculates the duration of daylight (time between sunrise and sunset) for a given
#' location and season on Mars. Implements Equation 10 from Appelbaum & Flood (1990).
#'
#' @param Ls Areocentric longitude [deg]
#' @param phi Planetary latitude [deg]
#'
#' @return Number of daylight hours [h]
#' @export
T_d = function(Ls, phi){
  
  # Equation 7 (1990): Declination angle [rad].
  delta = declination(Ls)
  
  # Equation 10 (1990).
  Td = (2/15) * (acos(-tan(phi*pi/180) * tan(delta))) * 180/pi

  return(Td)
}

tolerance = 0

# FIXME: Test against values in Table IV - Update 1990
# FIXME: Test against results from sunset - sunrise.
test_that("T_d", {
  for(Ls in 0){
    for(phi in seq(0, 80, 10)){
      Td = T_d(Ls=Ls, phi=phi)
      expect_equal(Td, 12, tolerance=tolerance, scale=1)
    }
  }
})

context("Beam daily insolation on an inclined surface")
source("utils.R")

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

test_that("H_bi: Beam daily insolation on an inclined surface for beta = phi at VL1", {

  # Tolerance.
  tolerance = 0.03

  # Test input parameters.
  Ls_seq = seq(0, 0, 5)

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL1",
    field = "Hb",
    tolerance = tolerance,
    Ls_seq = seq(0, 355, 5),
    beta_equals_phi = TRUE,
    verbose = FALSE)
})

test_that("H_bi: Beam daily insolation on optimal inclined surface for beta = 6.5° at VL1", {
  
  # Tolerance.
  tolerance = 0.03
  
  # Test input parameters.
  Ls_seq = seq(0, 355, 5) # Areocentric longitude.
  
  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL1",
    field = "Hb",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = FALSE,
    verbose = FALSE)
})


test_that("H_bi: Beam daily insolation on an inclined surface for beta = phi at VL2", {

  # Tolerance.
  tolerance = 0.057

  # Test input parameters.
  Ls_seq = seq(0, 360, 5)

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL2",
    field = "Hb",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = TRUE,
    verbose = FALSE)
})

test_that("H_bi: Beam daily insolation on optimal inclined surface for beta = 22° at VL2", {

  # Tolerance.
  tolerance = 0.08

  # Test input parameters.
  Ls_seq = seq(0, 355, 5) # Areocentric longitude.

  # Expect equals all calculations against all expected.
  test_daily_insolation_on_inclined_surface(
    spacecraft = "VL2",
    field = "Hb",
    tolerance = tolerance,
    Ls_seq = Ls_seq,
    beta_equals_phi = FALSE,
    verbose = FALSE)
})

test_that("H_bi: gamma_c validation", {
  # Test invalid gamma_c values (out of -180 to 180 range)
  expect_error(
    H_bi(Ls = 90, phi = 22.48, tau = 0.5, beta = 25, gamma_c = 181)
  )

  expect_error(
    H_bi(Ls = 90, phi = 22.48, tau = 0.5, beta = 25, gamma_c = -181)
  )
})

context("Normalized net flux function")

# Assert equals test function.
assert = function(x, al){
  net_flux = f(z=x["z"], tau=x["tau"], al=al)
  expect_equal(unname(x["netflux"]), net_flux, tolerance=0, scale=1)
}

test_that("f function: toggling warnings.", {
  Sys.setenv(NET_FLUX_FUNCTION_TYPE = "polynomial")
  
  # Enable warnings
  Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = TRUE)
  
  # Trigger warning regarding tau greater than 5.
  expect_warning(f(z=10, tau=6, al=0.1))
  
  # Trigger warning regarding z greater or equal than 80.
  expect_warning(f(z=85, tau=0.5, al=0.1))
  
  for(false_flag in c("1", "f", "false", "n", "no", "FaLsE", "FALSE", "F", "No", "NO", "N", FALSE)){
    # Disable warnings
    Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = false_flag)
    
    # Ignore warning regarding tau greater than 5.
    expect_silent(f(z=10, tau=6, al=0.1))
    
    # Ignore warning regarding z greater or equal than 80.
    expect_silent(f(z=85, tau=0.5, al=0.1))
  }
  
  # Enable warnings by setting show warnings flag to unrecognized values
  Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = "asdasd")
  
  # Trigger warning regarding tau greater than 5.
  expect_warning(f(z=10, tau=6, al=0.1))
  
  # Trigger warning regarding z greater or equal than 80.
  expect_warning(f(z=85, tau=0.5, al=0.1))
  

})

test_that("f function: comparing normalized flux lookup table values.", {
  # Test parameters.
  Z_seq = c(seq(0, 80, 10), 85)

  tau_seq = c(
    seq(0.1, 2.0, 0.1),
    seq(2.5, 3, 0.5),
    5, 6)

  al = 0.1 # Albedo.

  for(z in Z_seq){
    for(tau in tau_seq){

      # Calculate global irradiances.
      Sys.setenv(NET_FLUX_FUNCTION_TYPE = "lookup_v1")
      net_flux_lookup_v1 = f(z=z, tau=tau, al=al)

      Sys.setenv(NET_FLUX_FUNCTION_TYPE = "lookup_v2")
      net_flux_lookup_v2 = f(z=z, tau=tau, al=al)

      # Test assert equality.
      # Normalized net flux values in lookup tables lookup_v1 and lookup_v2 are not exactly equal all the time
      # but they are at least approximately equal.
      expect_equal(net_flux_lookup_v1, net_flux_lookup_v2, tolerance=0.051, scale=1)
    }
  }
})

test_that("f function - lookup_v2: albedo 0.1.", {

  # Expected results.
  expected_results = data.frame(
    "z" =       c(   0,  10,   15,   20,    25,   30,   35,  40,    45,   50),
    "tau" =     c(0.15, 0.55, 1.25, 1.75, 2.20, 3.00, 3.60, 4.50, 5.00, 6.00),
    "netflux" = c(.875, .802, .678, .592, .518, .411, .337, .257, .212, .154))

  # Apply test function.
  Sys.setenv(NET_FLUX_FUNCTION_TYPE = "lookup_v2")
  apply(expected_results, 1, assert, al=0.1)
})


test_that("f function - lookup_v2: albedo 0.4.", {

  # Expected results.
  expected_results = data.frame(
    "z" =       c(  40,   45,   50,    55,  60,   65,   70,    75,   80,   85),
    "tau" =     c(0.15, 0.55, 1.25, 1.75, 2.20, 3.00, 3.60, 4.50, 5.00, 6.00),
    "netflux" = c(.582, .519, .404, .324, .256, .182, .137, .099, .078, .054))


  # Apply test function.
  Sys.setenv(NET_FLUX_FUNCTION_TYPE = "lookup_v2")
  apply(expected_results, 1, assert, al=0.4)
})

test_that("f function - polynomial: albedo 0.1.", {

  # Test input parameters.
  z_seq = seq(0, 85, 5)

  tau_seq = c(
    seq(0.1, 2, 0.05)
    # TODO: seq(2.1, 3, 0.1),
    # TODO: seq(3.2, 4, 0.2)
    # TODO: seq(4.5, 6, 0.5)
  )

  al = 0.1 # Albedo

  # Analytical function should return net flux that is approximately equal to those found in the lookup tables
  for(z in z_seq){
    for(tau in tau_seq){

      Sys.setenv(NET_FLUX_FUNCTION_TYPE = "lookup_v2")
      net_flux_lookup = f(z=z, tau=tau, al=al)

      Sys.setenv(NET_FLUX_FUNCTION_TYPE = "polynomial")
      net_flux_polynomial = f(z=z, tau=tau, al=al)

      # Larger divergences for high values of Z.
      # Maximum error is 7% for Z = 80° or Z = 85°.
      # Adjust acceptale tolerance accordingly.
      tolerance = ifelse(z >= 65, 0.0471, 0.0171)

      expect_equal(net_flux_lookup, net_flux_polynomial, tolerance=tolerance, scale=1)
    }
  }
})


test_that("f function - polynomial: albedo 0.4.", {

  # Test input parameters.
  z_seq = seq(0, 85, 5)

  tau_seq = c(
    seq(0.1, 2, 0.05)
    # TODO: seq(2.1, 3, 0.1),
    # TODO: seq(3.2, 4, 0.2)
    # TODO: seq(4.5, 6, 0.5)
  )

  al = 0.4 # Albedo

  # Analytical function should return net flux that is approximately equal to those found in the lookup tables
  for(z in z_seq){
    for(tau in tau_seq){
      Sys.setenv(NET_FLUX_FUNCTION_TYPE = "lookup_v2")
      net_flux_lookup = f(z=z, tau=tau, al=al)

      Sys.setenv(NET_FLUX_FUNCTION_TYPE = "polynomial")
      net_flux_polynomial = f(z=z, tau=tau, al=al)

      # Larger divergences for high values of z.
      # Maximum error is 7% for z = 80° or z = 85°.
      # Adjust acceptale tolerance accordingly.
      tolerance = ifelse(z >= 65, 0.0471, 0.0171)

      expect_equal(net_flux_lookup, net_flux_polynomial, tolerance=tolerance, scale=1)
    }
  }
})

# Diffuse daily insolation on Mars horizontal surface [Wh/m2-day].
#
# Based on equations presented in the following publication:
#   Appelbaum, Joseph & Flood, Dennis. (1990). Solar radiation on Mars. Solar Energy. 45. 353–363. 10.1016/0038-092X(90)90156-7. 
#   https://ntrs.nasa.gov/?R=19890018252
#
# Expected results taken from:
#   TABLE VI. - HOURLY AND DAILY DIFFUSE INSOLATION O NA HORIZONTAL SURFACE AT MAR SURFACE.
#   TABLE V. Daily insolation on a horizontal surface (Update 1990).
context("Diffuse daily insolation on a horizontal surface")
source("utils.R")

# Disable warnings.
Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = FALSE)

# Test with expected results from TABLE VI.
test_that("H_dh: Diffuse daily insolation on a horizontal surface", {
  tolerance = 47

  phi = 22.3
  al = 0.1

  expected_results = list(
    "69" = c(0.65, 1572), # FIXME: Large error (47).
    "120" = c(0.40, 1326),
    "153" = c(0.50, 1574), # FIXME: Large error (21.6).
    "249" = c(1.40, 1586)) # FIXME: Large error (24.1).
    #"299" = c(3.25, 1012))  # FIXME: Large error (252)

  index = 1
  for(expected_result in expected_results){

    Ls = strtoi(names(expected_results)[index])
    tau = expected_result[1]
    Hdh_expected = expected_result[2]

    Hdh = H_dh(Ls=Ls, phi=phi, tau=tau, al=al)
    expect_equal(Hdh, Hdh_expected, tolerance=tolerance, scale=1)

    index = index + 1
  }
})

test_that("H_dh: Diffuse daily insolation on a horizontal surface at VL1", {
  # 3.5% Error tolerance.
  tol = 0.035

  Ls_seq = seq(0, 355, 5)
  al = 0.1

  # Run the tests.
  test_daily_insolation_on_horizontal_surface(
    spacecraft = "VL1",
    field = "Hdh",
    tolerance = tol,
    Ls_seq = Ls_seq,
    al = al,
    verbose = FALSE)

})

test_that("H_dh: Diffuse daily insolation on a horizontal surface at VL2", {
  # 3.8% Error tolerance.
  tol = 0.038
  al = 0.1

  Ls_seq = seq(0, 355, 5)

  # Run the tests.
  test_daily_insolation_on_horizontal_surface(
    spacecraft = "VL2",
    field = "Hdh",
    tolerance = tol,
    Ls_seq = Ls_seq,
    al = al,
    verbose = FALSE)

})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimal_angle.R
\name{optimal_angle}
\alias{optimal_angle}
\title{Optimal tilt angle for maximum daily insolation}
\usage{
optimal_angle(Ls, phi, unit = 1)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{unit}{Output unit: 1 for radians, 2 for degrees (default: 1)}
}
\value{
Optimal tilt angle [rad] or [deg] depending on unit parameter
}
\description{
Calculates the optimal surface tilt angle (beta) that maximizes daily solar energy collection
for a given location and season on Mars. The surface is assumed to face the equator.
Based on Equations 40 and 43 from Appelbaum (1993).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{df_albedo}
\alias{df_albedo}
\title{Albedo lookup table}
\format{
A data frame with albedo values indexed by latitude and longitude
}
\usage{
df_albedo
}
\description{
Lookup table for Mars surface albedo based on latitude and longitude.
Based on NASA Technical Memorandum data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_irradiated.R
\name{is_irradiated}
\alias{is_irradiated}
\title{Check if surface is receiving solar irradiance}
\usage{
is_irradiated(Ls, phi, Ts, z = Z(Ls, Ts, phi), beta = NULL, gamma_c = NULL)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{Ts}{Solar time [h]}

\item{z}{Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts}

\item{beta}{Surface tilt/slope angle from horizontal [deg]. Optional, for inclined surfaces}

\item{gamma_c}{Surface azimuth angle [deg]. Optional, for inclined surfaces}
}
\value{
TRUE if surface is receiving irradiance, FALSE otherwise
}
\description{
Determines whether a surface at a given location and time is receiving solar irradiance.
Accounts for polar night/day conditions, sunrise/sunset times, and sun position below horizon.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/I_bi.R
\name{I_bi}
\alias{I_bi}
\title{Beam insolation on Mars inclined surface over time period}
\usage{
I_bi(Ls, phi, tau, Ts_start, Ts_end, beta, gamma_c)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{Ts_start}{Start of integration period [h]}

\item{Ts_end}{End of integration period [h]}

\item{beta}{Surface tilt/slope angle from horizontal [deg]}

\item{gamma_c}{Surface azimuth angle [deg]. Zero facing equator, east negative, west positive (-180 to +180)}
}
\value{
Beam insolation on inclined surface over specified time period [Wh/m²]
}
\description{
Calculates the direct beam solar energy received on an inclined surface between specified
start and end times. Obtained by integrating beam irradiance over the time period.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_polar_night.R
\name{is_polar_night}
\alias{is_polar_night}
\title{Check if location is experiencing polar night}
\usage{
is_polar_night(Ls, phi)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}
}
\value{
TRUE if experiencing polar night, FALSE otherwise
}
\description{
Determines whether a given location on Mars is experiencing polar night (24-hour darkness)
for the specified season. During polar night, the sun remains below the horizon continuously.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H_bh.R
\name{H_bh}
\alias{H_bh}
\title{Beam daily insolation on Mars horizontal surface}
\usage{
H_bh(Ls, phi, tau)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{tau}{Atmospheric optical depth (dimensionless)}
}
\value{
Beam daily insolation [Wh/m²-day]
}
\description{
Calculates the direct beam solar energy received over a full Martian day on a horizontal surface.
Obtained by integrating beam irradiance from sunrise to sunset.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sunrise.R
\name{sunrise}
\alias{sunrise}
\title{Sunrise time on Mars}
\usage{
sunrise(Ls, phi, beta = NULL, gamma_c = NULL, unit = 1)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{beta}{Surface tilt angle [deg]. Optional, for inclined surfaces}

\item{gamma_c}{Surface azimuth angle [deg]. Optional, for inclined surfaces. Zero facing equator, east negative, west positive (-180 to +180)}

\item{unit}{Output unit: 1 for radians, 2 for degrees, 3 for solar hours (default: 1)}
}
\value{
Sunrise time [rad], [deg], or [h] depending on unit parameter, or NA during polar night/day
}
\description{
Calculates the sunrise time for a horizontal or inclined surface on Mars.
Returns NA during polar night/day periods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Z.R
\name{Z}
\alias{Z}
\title{Solar zenith angle}
\usage{
Z(Ls, phi, Ts)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{Ts}{Solar time [h]}
}
\value{
Sun zenith angle [deg]
}
\description{
Calculates the angle between the sun's rays and the vertical (zenith) direction.
A zenith angle of 0° means the sun is directly overhead, while 90° means the sun
is at the horizon. Implements Equation 6 from Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/declination.R
\name{declination}
\alias{declination}
\title{Solar declination angle on Mars}
\usage{
declination(Ls, unit = 1)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{unit}{Output unit: 1 for radians, 2 for degrees (default: 1)}
}
\value{
Declination angle [rad] or [deg] depending on unit parameter
}
\description{
Calculates the angular position of the Sun at solar noon with respect to the plane of the
Martian equator. For Mars: -24.936° <= delta <= 24.936°. The declination is 0° at vernal
(Ls=0°) and autumnal equinoxes (Ls=180°), +24.936° at summer solstice (Ls=90°), and -24.936°
at winter solstice (Ls=270°). Implements Equation 7 from Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optical_depth.R
\name{optical_depth}
\alias{optical_depth}
\title{Atmospheric optical depth on Mars}
\usage{
optical_depth(Ls, phi, model = 1)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{model}{Dust storm model (1 or 2). Model 1 assumes both storms originated at -30° latitude.
Model 2 assumes storms at -30° (Ls=215°) and -10° (Ls=295°)}
}
\value{
Atmospheric optical depth (dimensionless, minimum 0.5)
}
\description{
Calculates the optical depth of the Martian atmosphere as a function of latitude and season.
The optical depth varies spatially and temporally, with peaks during global dust storms.
Implements Equations 1 and 2 from Appelbaum, Landis & Sherman (1991).
}
\examples{
# Calculate optical depth at Viking Lander 1 site during dust storm season
tau <- optical_depth(Ls = 215, phi = 22.3, model = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/I_obh.R
\name{I_obh}
\alias{I_obh}
\title{Beam insolation at top of Mars atmosphere over time period}
\usage{
I_obh(Ls, phi, Ts_start, Ts_end)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{Ts_start}{Start of integration period [h]}

\item{Ts_end}{End of integration period [h]}
}
\value{
Beam insolation at top of atmosphere over specified time period [Wh/m²]
}
\description{
Calculates the solar beam energy on a horizontal surface at the top of the Martian atmosphere
(before atmospheric attenuation) between specified start and end times. Implements Equation 11
from Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/marsrad-package.R
\docType{package}
\name{marsrad-package}
\alias{marsrad}
\alias{marsrad-package}
\title{marsrad: Mars Solar Radiation}
\description{
A set of functions to calculate solar irradiance and insolation on Mars horizontal
and inclined surfaces. Based on NASA Technical Memoranda 102299, 103623, 105216,
106321, and 106700, i.e. the canonical Mars solar radiation papers.
}
\details{
The package provides three types of solar radiation calculations:
\itemize{
  \item Instantaneous irradiance (G_* functions) — Power per unit area at a specific moment
  \item Daily insolation (H_* functions) — Energy per unit area over one Martian sol
  \item Period insolation (I_* functions) — Energy per unit area over multiple sols
}

All calculations support both horizontal and inclined surfaces. The package includes
functions for optimal tilt angle calculation, sunrise/sunset times, and atmospheric
optical depth modeling.
}
\section{Package Information}{

\describe{
  \item{Version}{1.0.0}
  \item{License}{GPL-3}
  \item{Website}{\url{https://georges.fyi/marsrad/}}
  \item{Author}{\href{https://georges.fyi}{Georges Labrèche}}
  \item{Affiliation}{\href{https://tanagraspace.com}{Tanagra Space}}
  \item{Depends}{R (>= 2.10)}
}
}

\section{Citation}{

If you use this package in your research or publication, please cite the paper it was developed for:

Labrèche, G., & Cordes, F. (2020). Using a Rover's Active Suspension System as a 2-Axis Solar Tracker Mechanism.
\emph{15th International Symposium on Artificial Intelligence, Robotics and Automation in Space (i-SAIRAS '20)}.
\url{https://www.hou.usra.edu/meetings/isairas2020fullpapers/pdf/5035.pdf}
}

\section{Links}{

\describe{
  \item{Package website}{\url{https://georges.fyi/marsrad/}}
  \item{Package GitHub repository}{\url{https://github.com/georgeslabreche/marsrad}}
  \item{Author LinkedIn}{\url{https://www.linkedin.com/in/georgeslabreche/}}
  \item{Author website}{\url{https://georges.fyi}}
}
}

\examples{
# Calculate horizontal irradiance at Viking 1 landing site on Ls 90 (northern summer solstice)
G_h(Ls = 90, phi = 22.48, longitude = -48, Ts = 12, tau = 0.5)

# Find optimal panel tilt angle for the same location and season
optimal_angle(Ls = 90, phi = 22.48)

# Calculate daily insolation on a horizontal surface
H_h(Ls = 90, phi = 22.48, longitude = -48, tau = 0.5)

# Calculate insolation over a 24-hour period on an inclined surface
I_i(Ls = 90, phi = 22.48, longitude = -48, tau = 0.5, Ts_start = 0,
    Ts_end = 24, beta = 25, gamma_c = 0)

}
\references{
Appelbaum, J., & Flood, D. J. (1989). Solar Radiation on Mars.
NASA Technical Memorandum 102299.
\url{https://ntrs.nasa.gov/citations/19890018252}

Appelbaum, J., & Flood, D. J. (1990). Solar radiation on Mars: Update 1990.
NASA Technical Memorandum 103623.
\url{https://ntrs.nasa.gov/citations/19910005804}

Appelbaum, J., & Flood, D. J. (1991). Solar radiation on Mars: Update 1991.
NASA Technical Memorandum 105216.
\url{https://ntrs.nasa.gov/citations/19910023732}

Appelbaum, J., Sherman, I., & Landis, G. A. (1993).
Solar radiation on Mars: Stationary photovoltaic array.
NASA Technical Memorandum 106321.
\url{https://ntrs.nasa.gov/citations/19940010257}

Appelbaum, J., Flood, D. J., & Norambuena, M. (1994).
Solar radiation on Mars: Tracking photovoltaic array.
NASA Technical Memorandum 106700.
\url{https://ntrs.nasa.gov/citations/19950004977}
}
\seealso{
Labrèche, G. (2020). Exploiting the SherpaTT Rover Active Suspension System to Enable
Optimal Solar Array Inclination and Orientation for Long Traverses in a Martian Environment.
\emph{Master's Thesis, Luleå University of Technology}.
\url{https://www.diva-portal.org/smash/record.jsf?pid=diva2:1413245}
}
\author{
\strong{Maintainer}: Georges Labrèche \email{georges@tanagraspace.com} (\href{https://orcid.org/0000-0002-2104-8789}{ORCID})

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/G_obh.R
\name{G_obh}
\alias{G_obh}
\title{Beam irradiance on horizontal surface at top of Mars atmosphere}
\usage{
G_obh(Ls, phi = NULL, Ts = NULL, z = Z(Ls = Ls, phi = phi, Ts = Ts))
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]. Can be NULL if z is provided}

\item{Ts}{Solar time [h]. Can be NULL if z is provided}

\item{z}{Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts}
}
\value{
Beam irradiance at top of atmosphere [W/m²]
}
\description{
Calculates the solar beam irradiance on a horizontal surface at the top of the Martian
atmosphere (before atmospheric attenuation). Implements Equation 5 from Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H_di.R
\name{H_di}
\alias{H_di}
\title{Diffuse daily insolation on Mars inclined surface}
\usage{
H_di(
  Ls,
  phi,
  longitude,
  tau,
  al = albedo(latitude = phi, longitude = longitude, tau = tau),
  beta,
  gamma_c
)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{longitude}{Planetary longitude [deg]}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{al}{Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau}

\item{beta}{Surface tilt/slope angle from horizontal [deg]}

\item{gamma_c}{Surface azimuth angle [deg]. Zero facing equator, east negative, west positive (-180 to +180)}
}
\value{
Diffuse daily insolation on inclined surface [Wh/m²-day]
}
\description{
Calculates the diffuse solar energy (scattered by atmospheric dust) received over a full
Martian day on an inclined surface. Obtained by integrating diffuse irradiance from sunrise to sunset.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/G_dh.R
\name{G_dh}
\alias{G_dh}
\title{Diffuse irradiance on Mars horizontal surface}
\usage{
G_dh(
  Ls,
  phi,
  longitude,
  Ts = NULL,
  z = Z(Ls = Ls, phi = phi, Ts = Ts),
  tau,
  al = albedo(latitude = phi, longitude = longitude, tau = tau)
)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{longitude}{Planetary longitude [deg]}

\item{Ts}{Solar time [h]. Can be NULL for daily calculations}

\item{z}{Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{al}{Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau}
}
\value{
Diffuse irradiance on horizontal surface [W/m²]
}
\description{
Calculates the diffuse solar irradiance (scattered by atmospheric dust) incident on a
horizontal surface on Mars. Computed as the difference between global and direct beam
irradiance. Implements Equation 16 from Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/I_bh.R
\name{I_bh}
\alias{I_bh}
\title{Beam insolation on Mars horizontal surface over time period}
\usage{
I_bh(Ls, phi, tau, Ts_start, Ts_end)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{Ts_start}{Start of integration period [h]}

\item{Ts_end}{End of integration period [h]}
}
\value{
Beam insolation over specified time period [Wh/m²]
}
\description{
Calculates the direct beam solar energy received on a horizontal surface between specified
start and end times. Implements Equation 19 from Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f.R
\name{f}
\alias{f}
\title{Normalized net flux function}
\usage{
f(z, tau, al = 0.1)
}
\arguments{
\item{z}{Sun zenith angle [deg]}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{al}{Surface albedo (dimensionless, ranges from 0.1 to 0.4)}
}
\value{
Normalized net flux (dimensionless)
}
\description{
Calculates the normalized net solar flux on the Martian surface accounting for multiple
wavelength and multiple scattering in the atmosphere. Based on Pollack's calculations
presented in Appelbaum & Flood (1990). Can use polynomial expression or lookup tables.
}
\section{Environment Variables}{

The function behavior can be configured via environment variables:

\strong{NET_FLUX_FUNCTION_TYPE}: Controls which implementation to use:
\itemize{
  \item \code{"polynomial"} (default) - Analytical polynomial expression with ~0.7\% mean error.
        Maximum error ~7\% at zenith angles 80-85° and tau > 5.
  \item \code{"lookup_v1"} - Lookup table from NASA TM-102299. Albedo fixed at 0.1.
  \item \code{"lookup_v2"} - Lookup table from NASA TM-103623. Supports albedo 0.1 and 0.4.
}

Set with: \code{Sys.setenv(NET_FLUX_FUNCTION_TYPE = "polynomial")}

\strong{NET_FLUX_FUNCTION_SHOW_WARNINGS}: Controls warning display (TRUE/FALSE, default: TRUE).
Warnings are shown when polynomial calculations may have notable error margin (tau > 5 or z >= 80°).

Set with: \code{Sys.setenv(NET_FLUX_FUNCTION_SHOW_WARNINGS = TRUE)}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{df_netflux_k1_coeffs}
\alias{df_netflux_k1_coeffs}
\title{Net flux polynomial coefficients (k=1)}
\format{
A data frame with polynomial coefficients
}
\usage{
df_netflux_k1_coeffs
}
\description{
Polynomial coefficients for normalized net flux function calculation (albedo-dependent terms).
Based on Appelbaum & Flood analytical expression.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_polar_day.R
\name{is_polar_day}
\alias{is_polar_day}
\title{Check if location is experiencing polar day}
\usage{
is_polar_day(Ls, phi)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}
}
\value{
TRUE if experiencing polar day, FALSE otherwise
}
\description{
Determines whether a given location on Mars is experiencing polar day (24-hour sunlight)
for the specified season. During polar day, the sun remains above the horizon continuously.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/albedo.R
\name{albedo}
\alias{albedo}
\title{The albedo function.}
\usage{
albedo(latitude, longitude, tau, coordinates_rounding = TRUE)
}
\arguments{
\item{latitude}{Planetary latitude [deg], from -90° to +90°}

\item{longitude}{Planetary longitude [deg], from -180° to 180°}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{coordinates_rounding}{Logical, whether to round coordinates to nearest grid point}
}
\value{
Surface albedo value (dimensionless, 0-1)
}
\description{
Calculate the albedo value given geographical location and tau factor.
Source: Appelbaum, Joseph & Landis, Geoffrey & Sherman, I. (1991).  Solar radiation on Mars — Update 1991.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/G_ob.R
\name{G_ob}
\alias{G_ob}
\title{Beam irradiance at top of Martian atmosphere}
\usage{
G_ob(Ls)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}
}
\value{
Beam irradiance at top of atmosphere [W/m²]
}
\description{
Calculates the solar beam irradiance at the top of the Martian atmosphere (before any
atmospheric effects) as a function of Mars' orbital position. Accounts for Mars' elliptical
orbit which causes seasonal variation in solar intensity. Implements Equation 4 from
Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/T_d.R
\name{T_d}
\alias{T_d}
\title{Number of Mars daylight hours}
\usage{
T_d(Ls, phi)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}
}
\value{
Number of daylight hours [h]
}
\description{
Calculates the duration of daylight (time between sunrise and sunset) for a given
location and season on Mars. Implements Equation 10 from Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/G_bh.R
\name{G_bh}
\alias{G_bh}
\title{Direct beam irradiance on Mars horizontal surface}
\usage{
G_bh(Ls, phi, Ts, z = Z(Ls = Ls, phi = phi, Ts = Ts), tau)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{Ts}{Solar time [h]}

\item{z}{Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts}

\item{tau}{Atmospheric optical depth (dimensionless)}
}
\value{
Direct beam irradiance on horizontal surface [W/m²]
}
\description{
Calculates the direct beam solar irradiance incident on a horizontal surface on Mars.
Accounts for the angle of incidence on the horizontal plane. Implements Equation 18
from Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/I_dh.R
\name{I_dh}
\alias{I_dh}
\title{Diffuse insolation on Mars horizontal surface over time period}
\usage{
I_dh(
  Ls,
  phi,
  longitude,
  tau,
  Ts_start,
  Ts_end,
  al = albedo(latitude = phi, longitude = longitude, tau = tau)
)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{longitude}{Planetary longitude [deg]}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{Ts_start}{Start of integration period [h]}

\item{Ts_end}{End of integration period [h]}

\item{al}{Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau}
}
\value{
Diffuse insolation over specified time period [Wh/m²]
}
\description{
Calculates the diffuse solar energy (scattered by atmospheric dust) received on a horizontal
surface between specified start and end times. Computed as difference between global and beam insolation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/G_di.R
\name{G_di}
\alias{G_di}
\title{Diffuse irradiance on Mars inclined surface}
\usage{
G_di(
  Ls,
  phi,
  longitude,
  Ts,
  z = Z(Ls = Ls, phi = phi, Ts = Ts),
  tau,
  al = albedo(latitude = phi, longitude = longitude, tau = tau),
  beta
)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{longitude}{Planetary longitude [deg]}

\item{Ts}{Solar time [h]}

\item{z}{Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{al}{Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau}

\item{beta}{Surface tilt/slope angle from horizontal [deg]}
}
\value{
Diffuse irradiance on inclined surface [W/m²]
}
\description{
Calculates the diffuse solar irradiance (scattered by atmospheric dust) incident on an
inclined surface on Mars. Accounts for the view factor of the sky from the tilted surface.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{df_netflux_0p1_lookup_v2}
\alias{df_netflux_0p1_lookup_v2}
\title{Net flux lookup table (albedo 0.1, version 2)}
\format{
A data frame with net flux values indexed by zenith angle and optical depth
}
\usage{
df_netflux_0p1_lookup_v2
}
\description{
Lookup table for normalized net flux function with albedo 0.1.
From NASA TM-103623 (Appelbaum & Flood, 1990).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/G_bi.R
\name{G_bi}
\alias{G_bi}
\title{Direct beam irradiance on Mars inclined surface}
\usage{
G_bi(Ls, phi, Ts, z = Z(Ls = Ls, phi = phi, Ts = Ts), tau, beta, gamma_c)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{Ts}{Solar time [h]}

\item{z}{Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{beta}{Surface tilt/slope angle from horizontal [deg]}

\item{gamma_c}{Surface azimuth angle [deg]. Zero facing equator, east negative, west positive (-180 to +180)}
}
\value{
Direct beam irradiance on inclined surface [W/m²]
}
\description{
Calculates the direct beam solar irradiance incident on an inclined surface on Mars.
Accounts for the sun's angle of incidence on the tilted and oriented surface. Based on
Appelbaum, Flood & Norambuena (1994).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/G_b.R
\name{G_b}
\alias{G_b}
\title{Direct beam irradiance on Mars surface normal to solar rays}
\usage{
G_b(Ls, phi = NULL, Ts = NULL, z = Z(Ls = Ls, phi = phi, Ts = Ts), tau)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]. Can be NULL if z is provided}

\item{Ts}{Solar time [h]. Can be NULL if z is provided}

\item{z}{Sun zenith angle [deg]. If not provided, calculated from Ls, phi, and Ts}

\item{tau}{Atmospheric optical depth (dimensionless)}
}
\value{
Direct beam irradiance normal to sun [W/m²]
}
\description{
Calculates the direct beam solar irradiance on the Martian surface normal to the solar rays
(i.e., perpendicular to the sun's direction). Uses Beer's law to account for atmospheric
attenuation. Implements Equation 14 from Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H_bi.R
\name{H_bi}
\alias{H_bi}
\title{Beam daily insolation on Mars inclined surface}
\usage{
H_bi(Ls, phi, tau, beta, gamma_c)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{beta}{Surface tilt/slope angle from horizontal [deg]}

\item{gamma_c}{Surface azimuth angle [deg]. Zero facing equator, east negative, west positive (-180 to +180)}
}
\value{
Beam daily insolation on inclined surface [Wh/m²-day]
}
\description{
Calculates the direct beam solar energy received over a full Martian day on an inclined surface.
Obtained by integrating beam irradiance from sunrise to sunset.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H_obh.R
\name{H_obh}
\alias{H_obh}
\title{Daily beam insolation at top of Mars atmosphere}
\usage{
H_obh(Ls, phi)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}
}
\value{
Daily beam insolation at top of atmosphere [Wh/m²-day]
}
\description{
Calculates the solar beam energy over a full Martian day on a horizontal surface at the top
of the Martian atmosphere (before atmospheric attenuation). Implements Equation 13 from
Appelbaum & Flood (1990).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H_h.R
\name{H_h}
\alias{H_h}
\title{Global daily insolation on Mars horizontal surface}
\usage{
H_h(
  Ls,
  phi,
  longitude,
  tau,
  al = albedo(latitude = phi, longitude = longitude, tau = tau)
)
}
\arguments{
\item{Ls}{Areocentric longitude [deg]}

\item{phi}{Planetary latitude [deg]}

\item{longitude}{Planetary longitude [deg]}

\item{tau}{Atmospheric optical depth (dimensionless)}

\item{al}{Surface albedo (dimensionless, 0-1). If not provided, calculated from latitude, longitude, and tau}
}
\value{
Global daily insolation [Wh/m²-day]
}
\description{
Calculates the total solar energy received over a full Martian day on a horizontal surface.
Obtained by integrating global hourly insolation from sunrise to sunset.
}
